/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef ASYNCHRONOUS_SIGNALS_SIGNAL_HPP
#define ASYNCHRONOUS_SIGNALS_SIGNAL_HPP 1

#include <asynchronous/signals/signal_fwd.hpp>

#include <asynchronous/signals/signal_base.hpp>
#include <asynchronous/signals/type/combiner.hpp>
#include <asynchronous/signals/type/signal_executor.hpp>

#include <mpl/enable_constructor.hpp>
#include <use_default.hpp>

namespace asynchronous
{

  namespace signals
  {

    template < typename Result
	       , typename ... Args
	       , typename Combiner
	       , typename SignalTag
	       , typename InvocationPolicy
	       , template < typename ... > class SlotContainer
	       >
    class signal <Result(Args...),Combiner,SignalTag,InvocationPolicy,SlotContainer>
      : public signal_base<Result(Args...),SlotContainer>
    {
      using self = signal;
      using base = signal_base<Result(Args...),SlotContainer>;

      using combiner_type = type::combiner_t<Result(Args...),Combiner>;
      using signal_executor_type = type::signal_executor_t<SignalTag,Result(Args...),Combiner,SignalTag>;

    public:
      signal () = default;

      template < typename ... T
		 , typename = mpl::enable_constructor_t<self,T...>
		 >
      explicit signal (T&& ... t)
	: executor_(std::forward<T>(t)...)
      {}

    public:
      template < typename ... T >
      auto operator () (T&& ... t) const
      {
	return executor_(this->begin(),this->end(),std::forward<T>(t)...);
      }

    public:
      void clear_diagnostics () { executor_.clear_diagnostics(); }
      auto get_diagnostics () const
      {
	auto future = executor_.get_diagnostics();
	auto result = future.get();
	return result;
      }

    private:
      signal_executor_type executor_;
    };

  } // namespace signals

} // namespace asynchronous

#endif
