/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef CORE_DOMAIN_COMMON_DOMAIN_HPP
#define CORE_DOMAIN_COMMON_DOMAIN_HPP 1

#include "core/domain/default_domain.hpp"
#include "traits/domain.hpp"

namespace core
{

  namespace domain
  {

    template < typename ... T >
    class common_domain;

    template < typename T >
    struct common_domain< T >
    {
      using type = traits::domain_t<T>;
    };

    template < typename T >
    struct common_domain< T,T >
    {
      using type = traits::domain_t<T>;
    };

    template < typename T >
    struct common_domain< T,default_domain >
    {
      using type = traits::domain_t<T>;
    };

    template < typename T >
    struct common_domain< default_domain,T >
    {
      using type = traits::domain_t<T>;
    };

    template <>
    struct common_domain< default_domain,default_domain >
    {
      using type = default_domain;
    };

    template < typename T, typename U, typename ... V >
    struct common_domain< T,U,V... >
      : common_domain<
          typename common_domain< traits::domain_t<T>,traits::domain_t<U> >::type
          , V...
      >
    {};

    template < typename ... T >
    using common_domain_t = typename common_domain<T...>::type;
    
  } // namespace domain

} // namespace core

#endif
