/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef DATAFRAME_DATAFRAME_INSERTING_POLICY_HPP
#define DATAFRAME_DATAFRAME_INSERTING_POLICY_HPP 1

namespace dataframe
{

  struct dataframe_inserting_policy
  {
    template < typename Headers, typename Data, typename Line, typename LineNumber >
    static void missing_fields (Headers& headers, Data& data, Line line, LineNumber)
    {
      using size_type = typename Headers::size_type;
      using element_type = typename Line::value_type;
      for (size_type j = 0; j < headers.size(); ++j)
	{
	  if (j < line.size())
	    data[j].push_back(line[j]);
	  else
	    {
	      data[j].push_back(element_type());
	    }
	}
    }
    
    template < typename Headers, typename Data, typename Line, typename LineNumber >
    static void unexpected_fields (Headers& headers, Data& data, Line line, LineNumber)
    {
      using size_type = typename Line::size_type;
      headers.resize(line.size(),"???");
      for (size_type j = 0; j < line.size(); ++j)
	{
	  if (j < data.size())
	    data[j].push_back(line[j]);
	  else
	    {
	      using field_type = typename Data::value_type;
	      field_type extra_field(data[0].size()-1);
	      extra_field.push_back(line[j]);
	      data.push_back(extra_field);
	    }
	}
    }
  };

} // namespace dataframe

#endif
