/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef DATAFRAME_DATAFRAME_WRITER_HPP
#define DATAFRAME_DATAFRAME_WRITER_HPP 1

#include <iosfwd>
#include <dataframe/row_proxy.hpp>

namespace dataframe
{

  struct dataframe_writer
  {
    template < typename Dataframe >
    std::ostream& operator () (std::ostream& os, Dataframe const& dataframe) const
    { return apply(os,dataframe); }

    template < typename Dataframe >
    static std::ostream& apply (std::ostream& os, Dataframe const& dataframe)
    { return write(os,dataframe); }

    template < typename Dataframe >
    static std::ostream& write (std::ostream& os, Dataframe const& dataframe)
    {
      write_headers(os,dataframe);
      write_data(os,dataframe);
      return os;
    }

  private:
    template < typename Dataframe >
    static std::ostream&
    write_headers (std::ostream& os, Dataframe const& dataframe)
    {
      auto headers = dataframe.headers();
      bool first = true;
      for (auto header : headers)
	{
	  if (!first)
	    os << delimiter(os).value();
	  os << header;
	  first = false;
	}
      os << std::endl;
      return os;
    }

    template < typename Dataframe >
    static std::ostream&
    write_data (std::ostream& os, Dataframe const& dataframe)
    {
      using size_type = typename Dataframe::size_type;
      for (size_type row = 0; row < dataframe.size(); ++row)
	write_data_line(os,dataframe,row);
      return os;
    }

    template < typename Dataframe, typename Row >
    static void
    write_data_line (std::ostream& os, Dataframe const& dataframe, Row row)
    {
      auto fields = dataframe.fields();
      auto proxy = make_row_proxy(dataframe,row);
      bool first = true;
      for (decltype(fields) j = 0; j < fields; ++j)
	{
	  if (!first)
	    os << delimiter(os).value();
	  os << proxy(j);
	  first = false;
	}
      os << std::endl;
    }
  };

} // namespace dataframe

#endif
