/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef DATAFRAME_DELIMITER_HPP
#define DATAFRAME_DELIMITER_HPP 1

#include <dataframe/manipulator_storage_base.hpp>

namespace dataframe
{

  namespace detail
  {

    template < typename Derived, typename T >
    class delimiter_manipulator_storage_policy
      : manipulator_storage_policy_base<Derived,T>
    {
      using base = manipulator_storage_policy_base<Derived,T>;

    public:
      static char initialize () { return '\t'; }

      static char initialize (std::ios_base& ios_base)
      { auto delimiter = base::reference(ios_base); return delimiter ? delimiter : '\t'; }

      static void assign (std::ios_base& ios_base, char delimiter)
      { base::reference(ios_base) = delimiter; }
    };

  } // namespace detail

  class delimiter_manipulator
    : public manipulator_storage_base< delimiter_manipulator
				       , char
				       , detail::delimiter_manipulator_storage_policy<delimiter_manipulator,char>
				       >
  {
    using policy = detail::delimiter_manipulator_storage_policy<delimiter_manipulator,char>;
    using base = manipulator_storage_base< delimiter_manipulator,char,policy >;
    
  public:
    template < typename ... Args >
    delimiter_manipulator (Args&& ... args)
      : base(std::forward<Args>(args)...)
    {}
  };

  delimiter_manipulator delimiter (char c) { return delimiter_manipulator(c); }
  delimiter_manipulator delimiter (std::ios_base& ios_base)
  { return delimiter_manipulator(ios_base); }
  
} // namespace dataframe

#endif
