/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef DATAFRAME_ELEMENT_HPP
#define DATAFRAME_ELEMENT_HPP 1

#include <boost/optional.hpp>
#include <observer_ptr.hpp>

namespace dataframe
{

  template < typename T, typename Dataframe, typename Row, typename Field >
  class element
  {
    using self = element;
    using dataframe_type = observer_ptr<Dataframe const>;
    using optional_element_type = boost::optional<T>;

  public:
    using result_type = T;

  public:
    element () = default;
    
    element (Dataframe const& dataframe, Row row, Field field)
      : dataframe_(make_observer(dataframe))
      , row_(row)
      , field_(field)
    {}

    template < typename ... U >
    auto operator () (U&& ...) const { return cache_element(); }

  private:
    auto& dataframe () { return *dataframe_; }
    auto const& dataframe () const { return *dataframe_; }

    auto cache_element () const
    {
      if (!element_)
	element_ = dataframe().element<T>(row_,field_).value();
      assert(element_);
      return *element_;
    }

  private:
    dataframe_type dataframe_;
    Row row_;
    Field field_;
    mutable optional_element_type element_;
  };

  template < typename T, typename Dataframe, typename Row, typename Field >
  auto make_element (Dataframe const& dataframe, Row row, Field field)
  {
    using element_type = element<T,Dataframe,Row,Field>;
    return element_type(dataframe,row,field);
  }

} // namespace dataframe

#endif
