/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef DATAFRAME_ELEMENT_TRANSFORM_ITERATOR_HPP
#define DATAFRAME_ELEMENT_TRANSFORM_ITERATOR_HPP 1

#include <boost/iterator/transform_iterator.hpp>
#include <dataframe/element_transform.hpp>

namespace dataframe
{

  template < typename T
	     , typename U = T
	     >
  class element_transform_iterator
  {
    using transform = element_transform<T,U>;

  public:
    template < typename Iterator >
    auto operator () (Iterator i) const { return apply(i); }

    template < typename Iterator >
    static auto apply (Iterator i)
    {
      using boost::make_transform_iterator;
      return make_transform_iterator(i,transform());
    }

  public:
    template < typename Iterator >
    struct result
    {
      using type = decltype(apply(Iterator()));
    };

    template < typename Iterator >
    using result_t = typename result<Iterator>::type;
  };

} // namespace dataframe

#endif
