/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef DATAFRAME_MISSING_DATA_HPP
#define DATAFRAME_MISSING_DATA_HPP 1

#include <string>
#include <boost/algorithm/string/regex.hpp>
#include <dataframe/manipulator_storage_base.hpp>

namespace dataframe
{

  class missing_data_pattern
    : public manipulator_storage_base< missing_data_pattern,std::pair<std::string,std::string> >
  {
    using base = manipulator_storage_base< missing_data_pattern,std::pair<std::string,std::string> >;
    
    auto regex () const { using regex_type = boost::regex; return regex_type(pattern()); }

  public:
    template < typename ... Args >
    missing_data_pattern (Args&& ... args)
      : base(std::forward<Args>(args)...)
    {}

    std::string pattern () const { return this->value().first; }
    std::string replacement () const { return this->value().second; }

    std::string transform (std::string s) const
    {
      if (pattern().empty())
	return s;
      return boost::algorithm::replace_all_regex_copy(s,regex(),replacement());
      return s;
    }
  };

  auto missing_data (std::string pattern) { return missing_data_pattern(std::make_pair(pattern,"")); }
  auto missing_data (std::string pattern, std::string replacement) { return missing_data_pattern(std::make_pair(pattern,replacement)); }
  auto missing_data (std::ios_base& ios_base) { return missing_data_pattern(ios_base); }
  
} // namespace dataframe

#endif
