/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef DATAFRAME_ROW_ITERATOR_HPP
#define DATAFRAME_ROW_ITERATOR_HPP 1

#include <boost/iterator/iterator_facade.hpp>
#include <dataframe/row_proxy.hpp>

namespace dataframe
{

  template < typename Dataframe >
  class row_iterator
    : public boost::iterator_facade< row_iterator<Dataframe>
				     , row_proxy<Dataframe>
				     , boost::random_access_traversal_tag
				     , row_proxy<Dataframe>
				     >
  {
    using self = row_iterator;

    using row_proxy_type = row_proxy<Dataframe>;

    using pointer = observer_ptr<Dataframe const>;
    using size_type = typename Dataframe::size_type;
    using header_type = typename Dataframe::header_type;

  public:
    row_iterator () = default;
    
    explicit row_iterator (Dataframe const& dataframe)
      : row_(dataframe)
    {}

    row_iterator (Dataframe const& dataframe, size_type row)
      : row_(dataframe,row)
    {}

  private:
    friend class boost::iterator_core_access;

    auto dereference () const { return row_; }
    bool equal (self const& other) const { return row_ == other.row_; }
    void increment () { row_.increment(); }
    void decrement () { row_.decrement(); }
    template < typename N >
    auto advance (N n) { row_advance(n); return *this; }
    auto distance_to (self other) { return row_.distance_to(other.row_); }

  private:
    row_proxy_type row_;
  };

  template < typename Dataframe >
  auto make_row_iterator (Dataframe const& dataframe)
  {
    using row_iterator_type = row_iterator<Dataframe>;
    return row_iterator_type(dataframe);
  }

  template < typename Dataframe, typename Row >
  auto make_row_iterator (Dataframe const& dataframe, Row row)
  {
    using row_iterator_type = row_iterator<Dataframe>;
    return row_iterator_type(dataframe,row);
  }

} // namespace dataframe

#endif
