/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef DATAFRAME_SEQUENCE_PROXY_HPP
#define DATAFRAME_SEQUENCE_PROXY_HPP 1

#include <type_traits>
#include <dataframe/element_proxy.hpp>
#include <dataframe/range_proxy.hpp>
#include <observer_ptr.hpp>

namespace dataframe
{

  template < typename Sequence
	     , typename Value = typename Sequence::value_type
	     >
  class sequence_proxy
    : public range_proxy<Sequence,Value>
  {
    using self = sequence_proxy;
    using base = range_proxy<Sequence,Value>;

    using value_type = typename base::value_type;

  public:
    sequence_proxy () = default;

    template < typename ... Args >
    explicit sequence_proxy (Args&& ... args)
      : base(std::forward<Args>(args)...)
    {}

    self& operator = (Sequence const& sequence)
    {
      this->value() = sequence;
      return *this;
    }

    self& operator = (Sequence&& sequence)
    {
      this->value() = std::move(sequence);
      return *this;
    }

    template < typename Index >
    auto at (Index i)
    { return make_element_proxy<Value>(this->range().at(i)); }

    template < typename Index >
    auto at (Index i) const
    { return make_element_proxy<Value>(this->range().at(i)); }

    template < typename V, typename Index >
    auto at (Index i)
    { return make_element_proxy<V>(this->range().at(i)); }

    template < typename V, typename Index >
    auto at (Index i) const
    { return make_element_proxy<V>(this->range().at(i)); }
  };

  template < typename Sequence, typename Value >
  class sequence_proxy< Sequence const,Value >
    : public range_proxy<Sequence const,Value >
  {
    using self = sequence_proxy;
    using base = range_proxy<Sequence const,Value>;

  public:
    sequence_proxy () = default;
    
    template < typename ... Args >
    explicit sequence_proxy (Args&& ... args)
      : base(std::forward<Args>(args)...)
    {}

    template < typename Index >
    auto at (Index i) const
    { return make_element_proxy<Value>(this->range().at(i)); }

    template < typename V, typename Index >
    auto at (Index i) const
    { return make_element_proxy<V>(this->range().at(i)); }
  };

  template < typename Value, typename Sequence >
  auto make_sequence_proxy (Sequence& sequence)
  {
    using sequence_proxy_type = sequence_proxy<Sequence,Value>;
    return sequence_proxy_type(sequence);
  }

  template < typename Value, typename Sequence >
  auto make_sequence_proxy (Sequence const& sequence)
  {
    using sequence_proxy_type = sequence_proxy<Sequence const,Value>;
    return sequence_proxy_type(sequence);
  }

} // namespace dataframe

#endif
