/*
 * Copyright (c) 2017 Gregory Penn.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

#ifndef GDAL_GEOMETRY_HANDLE_HPP
#define GDAL_GEOMETRY_HANDLE_HPP

#include <ogr_geometry.h>
#include <gdal/smartptr.hpp>

namespace gdal
{

  class geometry_handle
  {
    using self = geometry_handle;
    using pointer    = smartptr<OGRGeometry>;

  public:
    geometry_handle (pointer p): p_(p) {}

    auto wkt ()
    {
      char* char_ptr;
      p_->exportToWkt(&char_ptr);
      std::shared_ptr<char> wkt(char_ptr, [](auto char_ptr) { CPLFree(char_ptr); });
      return std::string(char_ptr);

    }

    auto srs ()
    {
      // todo: const
      auto p_raw = p_->getSpatialReference();
      smartptr<OGRSpatialReference>
        p(
          p_raw
          , p_
          , [] () {} // deleter
          , [p_raw] () { p_raw->Reference(); }
          , [p_raw] () { p_raw->Dereference(); }
          , [p_raw] () { return p_raw->GetReferenceCount(); }
          );
      return geographic_srs(p);
    }

    auto gdal_ptr () const { return p_; }

    explicit operator bool () { return static_cast<bool>(p_); }
    bool     operator ==   (self const& other) const { return p_->Equals(other.p_.get()); }

  private:
    pointer p_;
  };

  std::ostream& operator << (std::ostream& os, geometry_handle const& h)
  {
    char* p;
    h.gdal_ptr()->exportToWkt(&p);
    std::shared_ptr<char> wkt(p, [](auto p) { CPLFree(p); });

    return os << p;
  }

} // namespace gdal

#endif
