/*
 * Copyright (c) 2017 Gregory Penn.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

#ifndef GDAL_GRID_SRS_HPP
#define GDAL_GRID_SRS_HPP

#include <boost/operators.hpp>

namespace gdal
{

  class grid_srs
    : boost::equality_comparable<grid_srs>
  {
    using self = grid_srs;

  public:
    // todo: For rasters the origin as at the top left corner. How can we encode this?
    grid_srs () = default;

    grid_srs(int xmax, int ymax): xmax_(xmax), ymax_(ymax) {}

  public:
    int xmax () const { return xmax_; }
    int ymax () const { return ymax_; }

    template < typename coord_array >
    bool in_bounds (coord_array coords) { return in_bounds(coords[0], coords[1]); }

    bool in_bounds (int x, int y)
    {
      return (x >= 0) && (x < xmax_)
          && (y >= 0) && (y < ymax_);
    }

    bool operator == (self const& other) const // { return true; }
    {
      return (xmax_ == other.xmax_)
          && (ymax_ == other.ymax_);
    }

  private:
    int xmax_;
    int ymax_;
  };

  std::ostream& operator << (std::ostream& os, grid_srs const& grid)
  { return os << "Grid (xmax, ymax): (" << grid.xmax() << ", " << grid.ymax() << ")"; }

} // namespace gdal

#endif
