/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file
/// \brief Construct a strategy for a multinomial distribution
///

#ifndef GRAPH_MODEL_DISTRIBUTION_MULTINOMIAL_MULTINOMIAL_DISTRIBUTION_HPP
#define GRAPH_MODEL_DISTRIBUTION_MULTINOMIAL_MULTINOMIAL_DISTRIBUTION_HPP 1

#include <graph_model/distribution/multinomial/multinomial_strategy.hpp>
#include <graph_model/distribution/parameter/named_parameter.hpp>
#include <graph_model/distribution/parameter/parameter.hpp>
#include <graph_model/distribution/parameter/parameter_pack.hpp>
#include <graph_model/distribution/parameter/probability.hpp>
#include <graph_model/distribution/parameter/variate.hpp>

namespace graph_model
{

  namespace distribution
  {

    namespace multinomial
    {

      namespace detail
      {

	template < typename Tags, typename Defaults, typename Tagged, typename Untagged
		   , typename Value
		   , typename ... Parameters
		   >
	auto multinomial_distribution
	(parameter::parameter_pack<Tags,Defaults,Tagged,Untagged> pack,
	 Value&& value,
	 Parameters&& ... parameters);

	template < typename Tags, typename Defaults, typename Tagged, typename Untagged >
	auto multinomial_distribution
	(parameter::parameter_pack<Tags,Defaults,Tagged,Untagged> pack)
	{ return make_multinomial_strategy(pack); }

	template < typename Tags, typename Defaults, typename Tagged, typename Untagged
		   , typename Key, typename Value
		   , typename ... Parameters
		   >
	auto multinomial_distribution
	(parameter::parameter_pack<Tags,Defaults,Tagged,Untagged> pack,
	 parameter::named_parameter<Key,Value> parameter,
	 Parameters&& ... parameters)
	{
	  return multinomial_distribution
	    (pack.add(parameter),
	     std::forward<Parameters>(parameters)...);
	}

	template < typename Tags, typename Defaults, typename Tagged, typename Untagged
		   , typename Value
		   , typename ... Parameters
		   >
	auto multinomial_distribution
	(parameter::parameter_pack<Tags,Defaults,Tagged,Untagged> pack,
	 Value&& value,
	 Parameters&& ... parameters)
	{
	  return multinomial_distribution
	    (pack.add(std::forward<Value>(value)),
	     std::forward<Parameters>(parameters)...);
	}

      }	// namespace detail

      template < typename ... Parameters >
      auto multinomial_distribution (Parameters&& ... parameters)
      {
	using parameter::make_parameter;
	using parameter::make_parameter_pack;
	return detail::multinomial_distribution
	  (make_parameter_pack
	   (make_parameter<parameter::tag::probability>(),
	    make_parameter<parameter::tag::variate>()),
	   std::forward<Parameters>(parameters)...);
      }

    } // namespace multinomial

  } // namespace distribution

} // namespace graph_model

#endif
