/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_DISTRIBUTION_PARAMETER_NAMED_PARAMETER_HPP
#define GRAPH_MODEL_DISTRIBUTION_PARAMETER_NAMED_PARAMETER_HPP 1

#include <graph_model/vertex/expression_function.hpp>

namespace graph_model
{

  namespace distribution
  {

    namespace parameter
    {
      
      template < typename Tag, typename T >
      struct named_parameter
      {
	named_parameter () = default;
	named_parameter (T t) : t_(t) {}

	auto& operator () () const { return t_; }

      private:
	T t_;
      };

      template < typename Other, typename Tag, typename T >
      auto retag (named_parameter<Tag,T> p)
      { return named_parameter<Other,T>(p()); }

      template < typename Tag, typename T >
      auto make_named_parameter (T&& t)
      {
	using vertex::make_expression_function;
	auto f = make_expression_function(std::forward<T>(t));
	using function_type = decltype(f);
	using named_parameter_type = named_parameter<Tag,function_type>;
	return named_parameter_type(f);
      }

    } // namespace parameter

  } // namespace distribution

} // namespace graph_model

#endif
