/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_EDGE_FACTOR_OUT_EDGE_ITERATOR_HPP
#define GRAPH_MODEL_EDGE_FACTOR_OUT_EDGE_ITERATOR_HPP 1

#include <iosfwd>
#include <boost/iterator/iterator_facade.hpp>
#include <graph_model/edge/edge_descriptor.hpp>
#include <graph_model/vertex/vertex_descriptor.hpp>
// #include <graph_model/vertex/vertex.hpp>

namespace graph_model
{

  namespace edge
  {

    template < typename VertexDescriptor, typename VertexCollection >
    class out_edge_iterator
      : public boost::iterator_facade< out_edge_iterator<VertexDescriptor,VertexCollection>
				       , edge_descriptor<VertexDescriptor>
				       , boost::forward_traversal_tag
				       , edge_descriptor<VertexDescriptor>
				       >
    {
      using self = out_edge_iterator;
      using const_iterator = typename VertexCollection::const_iterator;
      using edge_descriptor = edge_descriptor<VertexDescriptor>;

    public:
      out_edge_iterator () = default;
      out_edge_iterator (VertexDescriptor source, const_iterator i, const_iterator end)
	: source_(source)
	, i_(i)
	, end_(end)
      {}
      out_edge_iterator (VertexDescriptor source, VertexCollection const& adjacent_vertices)
	: out_edge_iterator(source,adjacent_vertices.begin(),adjacent_vertices.end())
      {}

    private:
      friend boost::iterator_core_access;

      auto dereference () const { return edge_descriptor(source_,*i_); }

      void increment () { ++i_; }

      template < typename V, typename C >
      bool equal (out_edge_iterator<V,C> other) const
      {
	return (is_end() && other.is_end())
	  || (!is_end() && !other.is_end() && source_ == other.source_ && i_ == other.i_);
      }
  
    private:
      bool is_end () const { return source_.is_null_vertex() || i_ == end_; }
	  
    private:
      VertexDescriptor source_;
      const_iterator i_;
      const_iterator end_;
    };

  } // namespace edge

} // namespace graph_model

#endif
