/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_GRAPH_BFS_VISITOR_HPP
#define GRAPH_MODEL_GRAPH_BFS_VISITOR_HPP 1

namespace graph_model
{

  namespace graph
  {

    template < typename VertexVisitor, typename EdgeVisitor >
    struct bfs_visitor
    {
      bfs_visitor () = default;
      bfs_visitor (VertexVisitor v, EdgeVisitor e) : vertex_visitor_(v), edge_visitor_(e) {}

      template < typename VertexDescriptor, typename Graph >
      void initialize_vertex (VertexDescriptor, Graph const&) {}

      template < typename VertexDescriptor, typename Graph >
      void discover_vertex (VertexDescriptor v, Graph const& g) { vertex_visitor_(v,g); }

      template < typename VertexDescriptor, typename Graph >
      void examine_vertex (VertexDescriptor, Graph const&) {}

      template < typename EdgeDescriptor, typename Graph >
      void examine_edge (EdgeDescriptor e, Graph const& g) { edge_visitor_(e,g); }

      template < typename EdgeDescriptor, typename Graph >
      void tree_edge (EdgeDescriptor, Graph const&) {}

      template < typename EdgeDescriptor, typename Graph >
      void non_tree_edge (EdgeDescriptor, Graph const&) {}

      template < typename VertexDescriptor, typename Graph >
      void gray_target (VertexDescriptor, Graph const&) {}

      template < typename VertexDescriptor, typename Graph >
      void black_target (VertexDescriptor, Graph const&) {}

      template < typename VertexDescriptor, typename Graph >
      void finish_vertex (VertexDescriptor, Graph const&) {}

    private:
      VertexVisitor vertex_visitor_;
      EdgeVisitor edge_visitor_;
    };

    template < typename VertexVisitor >
    struct bfs_visitor< VertexVisitor,void >
    {
      bfs_visitor () = default;
      bfs_visitor (VertexVisitor v) : vertex_visitor_(v) {}

      template < typename VertexDescriptor, typename Graph >
      void initialize_vertex (VertexDescriptor, Graph const&) {}

      template < typename VertexDescriptor, typename Graph >
      void discover_vertex (VertexDescriptor v, Graph const& g) { vertex_visitor_(v,g); }

      template < typename VertexDescriptor, typename Graph >
      void examine_vertex (VertexDescriptor, Graph const&) {}

      template < typename EdgeDescriptor, typename Graph >
      void examine_edge (EdgeDescriptor, Graph const&) {}

      template < typename EdgeDescriptor, typename Graph >
      void tree_edge (EdgeDescriptor, Graph const&) {}

      template < typename EdgeDescriptor, typename Graph >
      void non_tree_edge (EdgeDescriptor, Graph const&) {}

      template < typename VertexDescriptor, typename Graph >
      void gray_target (VertexDescriptor, Graph const&) {}

      template < typename VertexDescriptor, typename Graph >
      void black_target (VertexDescriptor, Graph const&) {}

      template < typename VertexDescriptor, typename Graph >
      void finish_vertex (VertexDescriptor, Graph const&) {}

    private:
      VertexVisitor vertex_visitor_;
    };

    template < typename EdgeVisitor >
    struct bfs_visitor< void,EdgeVisitor >
    {
      bfs_visitor () = default;
      bfs_visitor (EdgeVisitor e) : edge_visitor_(e) {}

      template < typename VertexDescriptor, typename Graph >
      void initialize_vertex (VertexDescriptor, Graph const&) {}

      template < typename VertexDescriptor, typename Graph >
      void discover_vertex (VertexDescriptor, Graph const&) {}

      template < typename VertexDescriptor, typename Graph >
      void examine_vertex (VertexDescriptor, Graph const&) {}

      template < typename EdgeDescriptor, typename Graph >
      void examine_edge (EdgeDescriptor e, Graph const& g) { edge_visitor_(e,g); }

      template < typename EdgeDescriptor, typename Graph >
      void tree_edge (EdgeDescriptor, Graph const&) {}

      template < typename EdgeDescriptor, typename Graph >
      void non_tree_edge (EdgeDescriptor, Graph const&) {}

      template < typename VertexDescriptor, typename Graph >
      void gray_target (VertexDescriptor, Graph const&) {}

      template < typename VertexDescriptor, typename Graph >
      void black_target (VertexDescriptor, Graph const&) {}

      template < typename VertexDescriptor, typename Graph >
      void finish_vertex (VertexDescriptor, Graph const&) {}

    private:
      EdgeVisitor edge_visitor_;
    };

    template < typename VertexVisitor, typename EdgeVisitor >
    auto make_bfs_visitor ()
    { return bfs_visitor<VertexVisitor,EdgeVisitor>(); }
    
    template < typename VertexVisitor, typename EdgeVisitor >
    auto make_bfs_visitor (VertexVisitor v, EdgeVisitor e)
    { return bfs_visitor<VertexVisitor,EdgeVisitor>(v,e); }
    
  } // namespace graph

} // namespace graph_model

#endif
