/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_GRAPH_TRAITS_HPP
#define GRAPH_MODEL_GRAPH_TRAITS_HPP 1

#include <boost/graph/graph_traits.hpp>
#include <graph_model/graph_model_fwd.hpp>
#include <graph_model/traits/adjacency_iterator.hpp>
#include <graph_model/traits/edge_descriptor.hpp>
#include <graph_model/traits/directed_category.hpp>
#include <graph_model/traits/edge_parallel_category.hpp>
#include <graph_model/traits/traversal_category.hpp>
#include <graph_model/traits/out_edge_iterator.hpp>
#include <graph_model/traits/degree_size.hpp>
#include <graph_model/traits/in_edge_iterator.hpp>
#include <graph_model/traits/adjacency_iterator.hpp>
#include <graph_model/traits/vertex_iterator.hpp>
#include <graph_model/traits/vertices_size.hpp>
#include <graph_model/traits/edge_iterator.hpp>
#include <graph_model/traits/edges_size.hpp>

namespace boost
{

  template < typename GraphTag >
  struct graph_traits< graph_model::graph_model<GraphTag> >
  {
    // graph concept
    using vertex_descriptor = graph_model::traits::vertex_descriptor_t<GraphTag>;
    using edge_descriptor = graph_model::traits::edge_descriptor_t<GraphTag>;
    using directed_category = graph_model::traits::directed_category_t<GraphTag>;
    using edge_parallel_category = graph_model::traits::edge_parallel_category_t<GraphTag>;
    using traversal_category = graph_model::traits::traversal_category_t<GraphTag>;

    // incidence graph concept
    using out_edge_iterator = graph_model::traits::out_edge_iterator_t<GraphTag>;
    using degree_size_type = graph_model::traits::degree_size_t<GraphTag>;

    // bidirectional graph concept
    using in_edge_iterator = graph_model::traits::in_edge_iterator_t<GraphTag>;

    // adjacency graph concept
    using adjacency_iterator = graph_model::traits::adjacency_iterator_t<GraphTag>;

    // vertex list graph concept
    using vertex_iterator = graph_model::traits::vertex_iterator_t<GraphTag>;
    using vertices_size_type = graph_model::traits::vertices_size_t<GraphTag>;

    // edge list graph concept
    using edge_iterator = graph_model::traits::edge_iterator_t<GraphTag>;
    using edges_size_type = graph_model::traits::edges_size_t<GraphTag>;
  };  

} // namespace boost

#endif
