/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_RANDOM_RANDOM_NUMBER_GENERATOR_HPP
#define GRAPH_MODEL_RANDOM_RANDOM_NUMBER_GENERATOR_HPP 1

#include <boost/random/mersenne_twister.hpp>

namespace graph_model
{

  namespace random
  {

    class random_number_generator
    {
      using generator_type = boost::mt19937;

    public:
      static generator_type& apply () { return generator(); }

      template < typename ... Args >
      static void seed (Args&& ... args) { generator().seed(std::forward<Args>(args)...); }
      
      static void reset  () { generator() = make_generator(); }

    private:
      static generator_type& generator ()
      {
	static generator_type generator = make_generator();
	return generator;
      }

      static generator_type make_generator () { return generator_type(); }      
    };

  } // namespace random

} // namespace graph_model

#endif
