/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_SIGNALS_TYPE_INVOCATION_POLICY_HPP
#define GRAPH_MODEL_SIGNALS_TYPE_INVOCATION_POLICY_HPP 1

#include <functional>
#include <graph_model/signals/type/invocation_result.hpp>
#include <use_default.hpp>

namespace graph_model
{

  namespace signals
  {

    namespace type
    {

      namespace detail
      {

	template < typename Type, typename Combiner, typename SlotIterator, typename ... Args >
	class invocation_policy
	{
	  using result_type = invocation_result_t<Combiner,SlotIterator,Args...>;
	  using function_type = std::function<result_type(Combiner,SlotIterator,SlotIterator,Args...)>;

	public:
	  using type = function_type;
	};

	template < typename Combiner, typename SlotIterator, typename ... Args >
	class invocation_policy< void,Combiner,SlotIterator,Args... >
	{
	  using result_type = invocation_result_t<void,SlotIterator,Args...>;
	  using function_type = std::function<result_type(SlotIterator,SlotIterator,Args...)>;

	public:
	  using type = function_type;
	};

      }	// namespace detail

      template < typename Combiner, typename SlotIterator, typename ... Args >
      struct invocation_policy
	: detail::invocation_policy< typename Combiner::type,Combiner,SlotIterator,Args...>
      {};

      template < typename SlotIterator, typename ... Args >
      struct invocation_policy<void,SlotIterator,Args...>
	: detail::invocation_policy< void,void,SlotIterator,Args...>
      {};

      template < typename Combiner, typename SlotIterator, typename ... Args >
      using invocation_policy_t = typename invocation_policy<Combiner,SlotIterator,Args...>::type;

    } // namecpace type

  } // namespace signals

} // namespace graph_model

#endif
