/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_FACTOR_BASE_HPP
#define GRAPH_MODEL_VERTEX_FACTOR_BASE_HPP 1

#include <graph_model/vertex/expression_base.hpp>
#include <graph_model/vertex/pdf_base.hpp>
#include <mpl/enable_constructor.hpp>

namespace graph_model
{

  namespace vertex
  {

    template < typename Probability >
    class factor_base
      : public expression_base<Probability>
      , public pdf_base<Probability>
    {
      using self = factor_base;
      using base = expression_base<Probability>;

    public:
      factor_base () = default;
      factor_base (factor_base&&) = default;
      factor_base& operator = (factor_base&&) = default;
      virtual ~factor_base () = default;

      template < typename ... Args
		 , typename = mpl::enable_constructor_t<self,Args...>
		 >
      explicit factor_base (Args&& ... args)
	: base(std::forward<Args>(args)...)
      {}
    };

  } // namespace vertex

} // namespace graph_model

#endif
