/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_IDENTIFIER_BASE_HPP
#define GRAPH_MODEL_VERTEX_IDENTIFIER_BASE_HPP 1

#include <type_traits>
#include <boost/lexical_cast.hpp>
#include <graph_model/vertex/type/identifier_base.hpp>
#include <use_default.hpp>

namespace graph_model
{

  namespace vertex
  {

    template < typename Identifier = use_default<> >
    class identifier_base
      : public virtual type::identifier_base<Identifier>
    {
      using base = type::identifier_base<Identifier>;

    public:
      auto identifier () const { return identifier_; }

      template < typename I, typename ... Is >
      void reset_identifier (I identifier, Is ... identifiers)
      {
	identifier_.clear();
	reset_identifier_impl(identifier,identifiers...);
      }

    private:
      void reset_identifier_impl () {}

      template < typename I, typename ... Is >
      void reset_identifier_impl (I identifier, Is ... identifiers)
      {
	using identifier_type = typename base::identifier_type;
	using is_convertible = typename std::is_convertible<I,identifier_type>::type;
	reset_identifier_impl(identifier,is_convertible());
	reset_identifier_impl(identifiers...);
      }

      template < typename I >
      void reset_identifier_impl (I identifier, std::false_type)
      {
	using identifier_type = typename base::identifier_type;
	identifier_ += boost::lexical_cast<identifier_type>(identifier);
      }

      template < typename I >
      void reset_identifier_impl (I identifier, std::true_type)
      { identifier_ += identifier; }

    private:
      typename base::identifier_type identifier_;
    };

  } // namespace vertex

} // namespace graph_model

#endif
