/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_MARKOV_BLANKET_BASE_HPP
#define GRAPH_MODEL_VERTEX_MARKOV_BLANKET_BASE_HPP 1

#include <cstddef>
#include <graph_model/vertex/type/probability_base.hpp>

namespace graph_model
{

  namespace vertex
  {

    template < typename Probability >
    class markov_blanket_base
      : public virtual type::probability_base<Probability>
    {
      using base = type::probability_base<Probability>;

    protected:
      using markov_blanket_size_type = std::size_t;

    public:
      auto markov_blanket_size () const { return markov_blanket_size_(); }
      auto markov_blanket () const { return markov_blanket_(); }

    private:
      virtual markov_blanket_size_type markov_blanket_size_ () const = 0;
      virtual typename base::probability_type markov_blanket_ () const = 0;
    };

  } // namespace vertex

} // namespace graph_model

#endif
