/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_RANDOM_VARIABLE_HPP
#define GRAPH_MODEL_VERTEX_RANDOM_VARIABLE_HPP 1

#include <graph_model/vertex/random_variable_fwd.hpp>
#include <boost/yap/algorithm.hpp>
#include <graph_model/vertex/distribution_base.hpp>
#include <graph_model/vertex/distribution_observer_base.hpp>
#include <graph_model/vertex/expression_terminal.hpp>
#include <graph_model/vertex/random_variable_base.hpp>
#include <graph_model/vertex/random_variable_impl_base.hpp>
#include <graph_model/vertex/tags.hpp>
#include <graph_model/vertex/vertex_expression.hpp>
#include <mpl/enable_constructor.hpp>

namespace graph_model
{

  namespace vertex
  {

    /// A concrete random variable vertex
    ///
    template < typename Variate, typename Probability >
    class random_variable
      : public random_variable_impl_base< random_variable<Variate,Probability>
					  , tag::random_variable
					  , random_variable_base<Variate,Probability>
					  >
      , public distribution_observer_base< random_variable<Variate,Probability>
					   , distribution_base<Variate,Probability>
					   >
    {};

    template < typename Variate, typename Probability >
    void swap (random_variable<Variate,Probability>& left, random_variable<Variate,Probability>& right)
    {
      assert(false);
    }

    template < typename ... T, typename ... Args >
    auto transform_expression (expression_terminal<random_variable<T...>> expr, Args&& ... args)
    { return evaluate_terminal(expr,std::forward<Args>(args)...); }

    template < typename ... T, typename ... Args >
    auto transform_expression (expression_terminal<random_variable<T...>&> expr, Args&& ... args)
    { return evaluate_terminal(expr,std::forward<Args>(args)...); }
					   
    template < typename Variate, typename Probability, typename ... Args >
    auto make_random_variable (Args&& ... args)
    {
      using random_variable_type = random_variable<Variate,Probability>;
      return random_variable_type(std::forward<Args>(args)...);
    }

    template < typename Probability, typename Variate, typename ... Args >
    auto make_random_variable (Variate variate, Args&& ... args)
    {
      using random_variable_type = random_variable<Variate,Probability>;
      random_variable_type rv(std::forward<Args>(args)...);
      rv.reset_variate(variate);
      return rv;
    }

    template < typename Variate, typename ... Args >
    auto make_random_variable (Args&& ... args)
    { return make_random_variable<Variate,double>(std::forward<Args>(args)...); }

    template < typename Variate, typename ... Args >
    auto make_random_variable (Variate variate, Args&& ... args)
    { return make_random_variable<double>(variate,std::forward<Args>(args)...); }

  } // namespace vertex

} // namespace graph_model

#endif
