/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_SAMPLE_HPP
#define GRAPH_MODEL_VERTEX_SAMPLE_HPP 1

#include <boost/yap/algorithm.hpp>
#include <graph_model/vertex/expression_terminal.hpp>
#include <graph_model/vertex/rvalue_reference_wrapper.hpp>
#include <graph_model/vertex/vertex_expression.hpp>
#include <mpl/enable_constructor.hpp>

namespace graph_model
{

  namespace vertex
  {

    template < typename Wrapper >
    class sample_wrapper
    {
      using self = sample_wrapper;

    public:
      template < typename W
		 , typename = mpl::enable_constructor_t<self,W>
		 >
      explicit sample_wrapper (W&& wrapper)
	: wrapper_(std::forward<W>(wrapper))
      {}

      template < typename ... T >
      auto operator () (T&& ... t) const { return wrapper_.get().sample(std::forward<T>(t)...); }

    private:
      Wrapper wrapper_;
    };

    template < typename ... T, typename ... Args >
    auto transform_expression (expression_terminal<sample_wrapper<T...>> expr, Args&& ... args)
    { return evaluate_terminal(expr,std::forward<Args>(args)...); }
					   
    template < typename ... T, typename ... Args >
    auto transform_expression (expression_terminal<sample_wrapper<T...>&> expr, Args&& ... args)
    { return evaluate_terminal(expr,std::forward<Args>(args)...); }

    template < typename Expr >
    auto make_sample_wrapper (std::reference_wrapper<Expr>&& wrapper)
    {
      using wrapper_type = std::reference_wrapper<Expr>;
      using function_type = sample_wrapper<wrapper_type>;
      return function_type(wrapper);
    }

    template < typename Expr >
    auto make_sample_wrapper (rvalue_reference_wrapper<Expr>&& wrapper)
    {
      using wrapper_type = rvalue_reference_wrapper<Expr>;
      using function_type = sample_wrapper<wrapper_type>;
      return function_type(wrapper);
    }

    template < typename Expr >
    auto make_sample_wrapper (Expr&& expr)
    {
      using std::cref;
      using ::graph_model::vertex::cref;
      return make_sample_wrapper(cref(std::forward<Expr>(expr)));
    }

    template < typename Expr >
    auto sample (Expr&& expr)
    {
      return make_sample_wrapper(std::forward<Expr>(expr));
    }

    template < typename Wrapper >
    auto sample (sample_wrapper<Wrapper> expr) { return expr; }

  } // namespace vertex

} // namespace graph_model

#endif
