/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_SCALAR_DISTRIBUTION_HPP
#define GRAPH_MODEL_VERTEX_SCALAR_DISTRIBUTION_HPP 1

#include <graph_model/vertex/scalar_distribution_fwd.hpp>
#include <boost/yap/algorithm.hpp>
#include <graph_model/vertex/expression_terminal.hpp>
#include <graph_model/vertex/strategy/basic_scalar_distribution.hpp>
#include <graph_model/vertex/scalar_distribution_base.hpp>
#include <graph_model/vertex/scalar_distribution_impl_base.hpp>
#include <graph_model/vertex/strategy_owner_base.hpp>
#include <graph_model/vertex/tags.hpp>
#include <graph_model/vertex/vertex_expression.hpp>

namespace graph_model
{

  namespace vertex
  {

    template < typename Variate, typename Probability >
    struct scalar_distribution
      : scalar_distribution_impl_base< scalar_distribution<Variate,Probability>
				       , tag::scalar_distribution
				       , Variate
				       , scalar_distribution_base<Variate,Probability>
				       >
      , public strategy_owner_base< scalar_distribution<Variate,Probability>
				    , strategy::basic_scalar_distribution_factory<Variate,Probability>
				    >
    {};

    template < typename Variate, typename Probability >
    void swap (scalar_distribution<Variate,Probability>& left, scalar_distribution<Variate,Probability>& right)
    {
      assert(false);
    }

    template < typename ... T, typename ... Args >
    auto transform_expression (expression_terminal<scalar_distribution<T...>> expr, Args&& ... args)
    { return evaluate_terminal(expr,std::forward<Args>(args)...); }
					   
    template < typename ... T, typename ... Args >
    auto transform_expression (expression_terminal<scalar_distribution<T...>&> expr, Args&& ... args)
    { return evaluate_terminal(expr,std::forward<Args>(args)...); }
					   
  } // namespace vertex

} // namespace graph_model

#endif
