/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_SCALAR_FACTOR_HPP
#define GRAPH_MODEL_VERTEX_SCALAR_FACTOR_HPP 1

#include <graph_model/vertex/scalar_factor_fwd.hpp>
#include <boost/yap/algorithm.hpp>
#include <graph_model/vertex/expression_terminal.hpp>
#include <graph_model/vertex/strategy/basic_scalar_factor.hpp>
#include <graph_model/vertex/scalar_factor_base.hpp>
#include <graph_model/vertex/scalar_factor_impl_base.hpp>
#include <graph_model/vertex/strategy_owner_base.hpp>
#include <graph_model/vertex/tags.hpp>
#include <graph_model/vertex/vertex_expression.hpp>

namespace graph_model
{

  namespace vertex
  {

    template < typename Variate, typename Probability >
    struct scalar_factor
      : scalar_factor_impl_base< scalar_factor<Variate,Probability>
				 , tag::scalar_factor
				 , Variate
				 , scalar_factor_base<Variate,Probability >
				 >
      , public strategy_owner_base< scalar_factor<Variate,Probability>
				    , strategy::basic_scalar_factor_factory<Variate,Probability>
				    >
    {};

    template < typename ... T, typename ... Args >
    auto transform_expression (expression_terminal<scalar_factor<T...>> expr, Args&& ... args)
    { return evaluate_terminal(expr,std::forward<Args>(args)...); }
					   
    template < typename ... T, typename ... Args >
    auto transform_expression (expression_terminal<scalar_factor<T...>&> expr, Args&& ... args)
    { return evaluate_terminal(expr,std::forward<Args>(args)...); }
					   
  } // namespace vertex

} // namespace graph_model

#endif
