/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_STRATEGY_MAKE_STRATEGY_HPP
#define GRAPH_MODEL_VERTEX_STRATEGY_MAKE_STRATEGY_HPP 1

#include <graph_model/vertex/rvalue_reference_wrapper.hpp>

namespace graph_model
{

  namespace vertex
  {

    namespace strategy
    {

      template < typename StrategyWrapper >
      struct strategy_wrapper
      {
	strategy_wrapper () = default;
	strategy_wrapper (StrategyWrapper wrapper)
	  : wrapper_(wrapper)
	{}

	auto operator () () const { return get()(); }
	auto cdf () const { return get().cdf(); }
	auto pdf () const { return get().pdf(); }
	auto sample () const { return get().sample(); }

      private:
	auto& get () { return wrapper_.get(); }
	auto& get () const { return wrapper_.get(); }

      private:
	StrategyWrapper wrapper_;
      };

      template < typename Strategy >
      auto make_strategy (Strategy&& strategy)
      {
	using wrapper_type = decltype(cref(std::forward<Strategy>(strategy)));
	using strategy_wrapper_type = strategy_wrapper<wrapper_type>;
	return strategy_wrapper_type(cref(std::forward<Strategy>(strategy)));
      }
	
    } // namespace strategy

  } // namespace vertex

} // namespace graph_model

#endif
