/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_STRATEGY_POLICY_OWNER_BASE_HPP
#define GRAPH_MODEL_VERTEX_STRATEGY_POLICY_OWNER_BASE_HPP 1

namespace graph_model
{

  namespace vertex
  {

    namespace strategy
    {

      template < typename Derived, typename Policy >
      class policy_owner_base
      {
	using self = policy_owner_base;

      public:
	policy_owner_base () = default;
	explicit policy_owner_base (Policy const& policy) : policy_(policy) {}
	explicit policy_owner_base (Policy&& policy) : policy_(std::move(policy)) {}

	// XXX - for CRTP only
	auto& policy () { return policy_; }
	auto& policy () const { return policy_; }

      private:
	auto& derived () { return static_cast<Derived&>(*this); }
	auto& derived () const { return static_cast<Derived const&>(*this); }

      private:
	Policy policy_;
      };

    } // namespace strategy

  } // namespace vertex

} // namespace graph_model

#endif
