/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_TRAPPING_EVAL_BASE_HPP
#define GRAPH_MODEL_VERTEX_TRAPPING_EVAL_BASE_HPP 1

#include <boost/throw_exception.hpp>
#include <graph_model/vertex/exception/missing_strategy.hpp>

namespace graph_model
{

  namespace vertex
  {

    /// \brief Debugging base class
    ///
    /// The purpose of this base class is simply to override the
    /// eval_() member function in a way that traps calls and prints a
    /// stack trace.  This may aid debugging.
    ///
    template < typename Derived, typename Base >
    class trapping_eval_base
      : public Base
    {
      using base = Base;

      typename base::result_type eval_ () const override
      { 
	BOOST_THROW_EXCEPTION(exception::missing_strategy());
      }
    };

  } // namespace vertex

} // namespace graph_model

#endif
