/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_FACTOR_VERTEX_ITERATOR_HPP
#define GRAPH_MODEL_VERTEX_FACTOR_VERTEX_ITERATOR_HPP 1

#include <boost/iterator/iterator_facade.hpp>

namespace graph_model
{

  namespace vertex
  {

    template < typename VertexCollection >
    class vertex_iterator
      : public boost::iterator_facade< vertex_iterator<VertexCollection>
				       , decltype(*typename VertexCollection::const_iterator())
				       , boost::forward_traversal_tag
				       , decltype(*typename VertexCollection::const_iterator())&
				       >
    {
      using self = vertex_iterator;
      using const_iterator = typename VertexCollection::const_iterator;

    public:
      vertex_iterator () = default;
      vertex_iterator (const_iterator i, const_iterator end)
	: i_(i)
	, end_(end)
      {}
      vertex_iterator (VertexCollection const& vertices)
	: vertex_iterator(vertices.begin(),vertices.end())
      {}

    private:
      friend boost::iterator_core_access;
      auto& dereference () const { return *i_; }
      void increment () { ++i_; }
      bool equal (self other) const
      {
	return (is_end() && other.is_end())
	  || (!is_end() && !other.is_end() && i_ == other.i_);
      }

    private:
      bool is_end () const { return i_ == end_; }

    private:
      const_iterator i_;
      const_iterator end_;
    };

  } // namespace vertex

} // namespace graph_model

#endif
