/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef GRAPH_MODEL_VERTEX_VERTEX_OBSERVER_COLLECTION_BASE_HPP
#define GRAPH_MODEL_VERTEX_VERTEX_OBSERVER_COLLECTION_BASE_HPP 1

#include <cassert>
#include <algorithm>
#include <vector>
#include <graph_model/edge/out_edge_iterator.hpp>
#include <graph_model/vertex/observer_collection.hpp>
#include <observer_ptr.hpp>
#include <type_traits/is_rvalue.hpp>

#include <debugging/stack_trace.hpp>

namespace graph_model
{

  namespace vertex
  {

    template < typename Derived >
    class vertex_observer_collection_base
    {
    public:
      using observer_collection_type = observer_collection<Derived const>;
      using const_vertex_collection_iterator = typename observer_collection_type::const_iterator;

    private:
      observer_collection_type observers_;

    public:
      auto out_degree () const { return observers_.size(); }
      auto out_edges () const
      {
	using vertex_descriptor = decltype(derived().vertex_descriptor());
	using out_edge_iterator = edge::out_edge_iterator<vertex_descriptor,observer_collection_type>;
	return std::make_pair(out_edge_iterator(derived().vertex_descriptor(),observers_),out_edge_iterator());
      }

      template < typename Vertex >
      auto add_adjacent_vertex (Vertex const& vertex)
      { return observers_.insert(make_observer(vertex)); }

      template < typename Vertex
		 , typename = std::enable_if_t<::type_traits::is_rvalue_v<Vertex>>
		 >
      auto add_adjacent_vertex (Vertex&& vertex)
      {
	std::cout << debugging::stack_trace<>() << std::endl;
	assert(false);
      }

      template < typename T >
      bool remove_adjacent_vertex (observer_ptr<T> observer)
      { return observers_.remove(observer); }

    private:
      Derived& derived () { return static_cast<Derived&>(*this); }
      Derived const& derived () const { return static_cast<Derived const&>(*this); }
    };

  } // namespace vertex

} // namespace graph_model

#endif
