/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef LOGGING_INITIALIZE_CONSOLE_LOG_HPP
#define LOGGING_INITIALIZE_CONSOLE_LOG_HPP 1

#include <boost/log/utility/setup/common_attributes.hpp>
#include <boost/log/utility/setup/console.hpp>

namespace logging
{

  struct initialize_console_log
  {
    template < typename ... Args >
    static void apply (Args&& ... args)
    {
      static auto initialized = initialize(std::forward<Args>(args)...);
    }

  private:
    template < typename ... Args >
    static bool initialize (Args&& ... args)
    {
      auto sink = boost::log::add_console_log(std::forward<Args>(args)...);
      boost::log::add_common_attributes();
      return true;
    }
  };

} // namespace logging

#endif
