/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef SAMPLE_SPACE_ASSERT_IS_POSSIBLE_HPP
#define SAMPLE_SPACE_ASSERT_IS_POSSIBLE_HPP 1

#include <stdexcept>
#include <boost/lexical_cast.hpp>
#include <boost/throw_exception.hpp>
#include <debugging/stack_trace.hpp>
#include <mpl/enable_constructor.hpp>
#include <sample_space/is_possible.hpp>

namespace sample_space
{

  namespace detail
  {

    class value_is_not_possible
      : public std::range_error
      , public virtual boost::exception
    {
      using self = value_is_not_possible;

    public:
      value_is_not_possible ()
	: std::range_error("value is not possible")
      {}
      template < typename T
		 , typename = mpl::enable_constructor_t<self,T>
		 >
      value_is_not_possible (T t)
	: std::range_error(("value is not possible: " + boost::lexical_cast<std::string>(t)).c_str())
      {}
    };

  } // namespace detail

  template < typename T >
  auto assert_is_possible (T t)
  {
    using sample_space::is_possible;
    if (!is_possible(t))
      {
	std::cout << debugging::stack_trace<>() << std::endl;
	BOOST_THROW_EXCEPTION(detail::value_is_not_possible{t});
      }
  }

} // namespace sample_space

#endif
