/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef SAMPLE_SPACE_MATH_DIVIDE_ASSIGN_HPP
#define SAMPLE_SPACE_MATH_DIVIDE_ASSIGN_HPP 1

#include <boost/range/algorithm/for_each.hpp>
#include "type_traits/is_map.hpp"

namespace sample_space
{
  
  namespace math
  {

    namespace detail
    {

      template < typename T
		 , typename U
		 , typename = type_traits::is_map_t<T>
		 >
      class divide_assign;

      template < typename T, typename U >
      struct divide_assign< T,U,std::false_type >
      {
	static void apply (T& t, U u)
	{ using boost::for_each; for_each(t,[u](auto& x){ x /= u; }); }
      };
      
      template < typename T, typename U >
      struct divide_assign< T,U,std::true_type >
      {
	static void apply (T& t, U u)
	{ using boost::for_each; for_each(t,[u](auto& pair){ pair.second /= u; }); }
      };
      
    } // namespace detail
    
    template < typename T, typename U >
    void divide_assign (T& t, U u)
    { detail::divide_assign<T,U>::apply(t,u); }

  } // namespace math

} // namespace sample_space

#endif
