/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef SAMPLE_SPACE_MATH_MULTINOMIAL_PDF_HPP
#define SAMPLE_SPACE_MATH_MULTINOMIAL_PDF_HPP 1

#include <boost/operators.hpp>
#include "sample_space/math/categorical_pdf.hpp"
#include "sample_space/math/multinomial_coefficient.hpp"
#include "sample_space/math/multinomial_pdf_ratio.hpp"

namespace sample_space
{
  
  namespace math
  {

    template < typename Value, typename P, typename N >
    class multinomial_pdf
      : boost::equality_comparable<multinomial_pdf<Value,P,N>>
    {
      using self = multinomial_pdf;
      using categorical_pdf_type = categorical_pdf<P,N>;
      using multinomial_coefficient_type = multinomial_coefficient<Value,N>;

    public:
      using value_type = Value;
      
    public:
      multinomial_pdf () = default;
      
      explicit multinomial_pdf (P p, N n)
	: multinomial_coefficient_(n)
	, categorical_pdf_(p,n)
      {}

      operator value_type () const { return value(); }

      value_type value () const
      { return multinomial_coefficient_.value() * categorical_pdf_.value(); }

      friend auto operator / (self left, self right)
      { return make_multinomial_pdf_ratio(left,right); }

      friend bool operator == (self left, self right)
      {
	return left.multinomial_coefficient_ == right.multinomial_coefficient_
	  && left.categorical_pdf_ == right.categorical_pdf_;
      }

    private:
      multinomial_coefficient_type multinomial_coefficient_;
      categorical_pdf_type categorical_pdf_;
    };

    template < typename Value, typename P, typename N >
    auto make_multinomial_pdf (P p, N n)
    {
      using multinomial_pdf_type = multinomial_pdf<Value,P,N>;
      return multinomial_pdf_type(p,n);
    }

  } // namespace math

} // namespace sample_space

#endif
