/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef SAMPLE_SPACE_VECTOR_POLICY_ARRAY_STORAGE_POLICY_HPP
#define SAMPLE_SPACE_VECTOR_POLICY_ARRAY_STORAGE_POLICY_HPP 1

#include <array>
#include <cstddef>
#include <mpl/enable_constructor.hpp>
#include <traits/array.hpp>

namespace sample_space
{

  namespace vector
  {
    
    namespace policy
    {

      template < typename T, std::size_t Size >
      class array_storage_policy
      {
	using self = array_storage_policy;
	using array_type = std::array<T,Size>;

      public:
	using size_type = typename array_type::size_type;
	using iterator = typename array_type::iterator;
	using const_iterator = typename array_type::const_iterator;

      public:
	template < typename ... Args
		   , typename = ::mpl::enable_constructor_t<self,Args...>
		   >
	array_storage_policy (Args&& ... args)
	  : array_(std::forward<Args>(args)...)
	{}

	array_storage_policy (std::initializer_list<T> x)
	  : array_(x)
	{}

	bool empty () const { return array_.empty(); }
	size_type size () const { return array_.size(); }

	iterator begin () { return array_.begin(); }
	iterator end () { return array_.end(); }

	friend iterator begin (self& base) { return base.begin(); }
	friend iterator end (self& base) { return base.end(); }
	
	const_iterator begin () const { return array_.begin(); }
	const_iterator end () const { return array_.end(); }

	friend const_iterator begin (self const& base) { return base.begin(); }
	friend const_iterator end (self const& base) { return base.end(); }
	
	T& operator [] (size_type n) { return array_[n]; }
	T const& operator [] (size_type n) const { return array_[n]; }

	T& at (size_type n) { return array_[n]; }
	T const& at (size_type n) const { return array_[n]; }

	auto& zero () { array_.fill(0); return *this; }
	auto& one () { array_.fill(1); return *this; }

      private:
	array_type array_;
      };

    } // namespace policy

  } // namespace vector
  
} // namespace sample_space

#endif
