/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef SAMPLE_SPACE_VECTOR_SIMPLEX_HPP
#define SAMPLE_SPACE_VECTOR_SIMPLEX_HPP 1

#include <sample_space/vector/simplex_fwd.hpp>
#include <mpl/enable_constructor.hpp>
#include <sample_space/vector/vector_base.hpp>

namespace sample_space
{

  namespace vector
  {

    /// \brief A vector on a simplex
    /// \tparam T The value type
    ///
    template < typename T, typename Index >
    class simplex
      : public vector_base<simplex<T,Index>,T,Index,tag::simplex>
    {
      using self = simplex;
      using base = vector_base<self,T,Index,tag::simplex>;

    public:
      template < typename ... Args
		 , typename = ::mpl::enable_constructor_t<self,Args...>
		 >
      simplex (Args&& ... args)
	: base(std::forward<Args>(args)...)
      {}

      simplex (std::initializer_list<T> x)
	: base(x)
      {}
    };

    template < typename T, typename Index >
    auto zero (simplex<T,Index> s) { return s.zero(); }

    template < typename T, typename Index >
    auto one (simplex<T,Index> s) { return s.one(); }

    template < typename T, typename Index >
    auto variate (simplex<T,Index> x) { return x; }

    template < typename T, typename Index, typename Context >
    auto variate (simplex<T,Index> x, Context) { return x; }

  } // namespace vector
  
} // namespace sample_space

#endif
