/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef SAMPLE_SPACE_VECTOR_VECTOR_BASE_HPP
#define SAMPLE_SPACE_VECTOR_VECTOR_BASE_HPP 1

#include <algorithm>
#include <sample_space/vector/mpl/vector_base.hpp>
#include <mpl/enable_constructor.hpp>
#include <use_default.hpp>

namespace sample_space
{

  namespace vector
  {
    
    /// \brief A vector space class
    /// \tparam Derived The derived type for CRTP
    /// \tparam T The type of field
    /// \tparam Index The type of index for dimensions
    /// \tparam Tag The tag defining the type of vector space
    /// \tparam Basis The basis for the vector space
    /// \tparam Storage The type of storage for the vector
    ///
    template < typename Derived
	       , typename T
	       , typename Index = void
	       , typename Tag = use_default<>
	       , typename Basis = use_default<>
	       , typename Storage = use_default<>
	       >
    class vector_base
      : public mpl::vector_base_t<Derived,T,Index,Tag,Basis,Storage>
    {
      using self = vector_base;
      using base = mpl::vector_base_t<Derived,T,Index,Tag,Basis,Storage>;

    public:
      template < typename ... Args
		 , typename = ::mpl::enable_constructor_t<self,Args...>
		 >
      vector_base (Args&& ... args)
	: base(std::forward<Args>(args)...)
      {};

      vector_base (std::initializer_list<T> x)
	: base(x)
      {}
    };

    template < typename Derived, typename T, typename Tag, typename Basis, typename Storage, typename Value >
    auto find (vector_base<Derived,T,void,Tag,Basis,Storage> const& v, Value value)
    { return std::find(v.begin(),v.end(),value); }
    
    template < typename Derived, typename T, typename Index, typename Tag, typename Basis, typename Storage, typename Key >
    auto find (vector_base<Derived,T,Index,Tag,Basis,Storage> const& v, Key key)
    { return std::find_if(v.begin(),v.end(),[key](auto pair){ return pair.first == key; }); }
    
  } // namespace vector
  
} // namespace sample_space

#endif
