/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef TRAITS_DOMAIN_HPP
#define TRAITS_DOMAIN_HPP 1

#include <type_traits>
#include <core/domain/default_domain.hpp>
#include <type_traits/is_domain.hpp>

namespace traits
{

  namespace defaults
  {

    template < typename T
	       , typename = type_traits::is_domain_t<T>
	       >
    class domain;

    template < typename T >
    struct domain< T,std::true_type >
    {
      using type = T;
    };

    template < typename T >
    struct domain< T,std::false_type >
    {
      using type = core::domain::default_domain;
    };

  } // namespace defaults
    
  namespace customization
  {

    template < typename T >
    struct domain
      : defaults::domain<T>
    {};

  } // namespace customization

  template < typename T >
  struct domain
    : customization::domain<std::decay_t<T>>
  {};

  template < typename T >
  using domain_t = typename domain<T>::type;

} // namespace traits

#endif
