/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef TRAITS_PROBABILITY_HPP
#define TRAITS_PROBABILITY_HPP 1

#include <type_traits>
#include <traits/value.hpp>

namespace traits
{

  namespace defaults
  {
    
    template < typename T >
    class probability;

#define DEFINE_PROBABILITY(T,U)		\
    template <>				\
    struct probability<T>		\
    {					\
      using type = U;			\
    };					\
    /**/

    DEFINE_PROBABILITY(bool,double)
    DEFINE_PROBABILITY(char,double)
    DEFINE_PROBABILITY(signed char,double)
    DEFINE_PROBABILITY(int,double)
    DEFINE_PROBABILITY(short,double)
    DEFINE_PROBABILITY(long,double)
    DEFINE_PROBABILITY(long long,long double)

    DEFINE_PROBABILITY(unsigned char,double)
    DEFINE_PROBABILITY(unsigned int,double)
    DEFINE_PROBABILITY(unsigned short,double)
    DEFINE_PROBABILITY(unsigned long,double)
    DEFINE_PROBABILITY(unsigned long long,long double)

    DEFINE_PROBABILITY(float,float)
    DEFINE_PROBABILITY(double,double)
    DEFINE_PROBABILITY(long double,long double)

#undef DEFINE_PROBABILITY

  } // namespace defaults

  namespace customization
  {

    template < typename T >
    struct probability
      : defaults::probability<T>
    {};

  } // namespace customization

  template < typename T >
  struct probability
    : customization::probability<std::decay_t<T>>
  {};

  template < typename T >
  using probability_t = typename probability<T>::type;

} // namespace traits

#endif
