/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#ifndef TRAITS_REAL_HPP
#define TRAITS_REAL_HPP 1

#include <type_traits>
#include <boost/proto/core.hpp>

namespace traits
{

  namespace defaults
  {
    
    template < typename T
	       , typename = std::integral_constant<bool,boost::proto::is_expr<T>::value>
	       , typename = typename std::is_integral<T>::type
	       >
    class real;

    template < typename T >
    struct real< T,std::false_type,std::false_type >
    {
      using type = T;
    };

    template < typename T >
    struct real< T,std::false_type,std::true_type >
    {
      using type = double;
    };

    template <>
    struct real< long long,std::false_type,std::true_type >
    {
      using type = long double;
    };

    template < typename T >
    struct real< T,std::true_type,std::false_type >
    {
      using type = double;
    };

  } // namespace defaults

  namespace customization
  {

    template < typename T >
    struct real
      : defaults::real<T>
    {};

  } // namespace customization

  template < typename T >
  struct real
    : customization::real<std::decay_t<T>>
  {};

  template < typename T >
  using real_t = typename real<T>::type;

} // namespace traits

#endif
