/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#define BOOST_TEST_MODULE dataframe_element

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <dataframe/element.hpp>
#include <dataframe/dataframe.hpp>
#include <dataframe/read_dataframe.hpp>
#include <boost/filesystem/fstream.hpp>
#include <boost/mpl/vector.hpp>

using test_types = boost::mpl::vector<int,long,float,double>;

BOOST_AUTO_TEST_CASE_TEMPLATE(test_element,T,test_types)
{
  using dataframe_type = dataframe::dataframe<>;
  using ifstream_type = boost::filesystem::ifstream;

  using dataframe::comment;
  using dataframe::make_element;
  using dataframe::skip;
  using dataframe::skip_blank_lines;
  using dataframe::trim_fields;
  using dataframe::trim_whitespace;
  
  ifstream_type is("test.dat");

  dataframe_type dataframe;
  is >> comment('#')
     >> skip(2)
     >> skip_blank_lines()
     >> trim_fields()
     >> trim_whitespace()
     >> dataframe;

  {
    auto element = make_element<T>(dataframe,0,"value");
    BOOST_CHECK_EQUAL(element(),1);
  }

  {
    auto element = make_element<T>(dataframe,1,"value");
    BOOST_CHECK_EQUAL(element(),-2);
  }

  {
    auto element = make_element<T>(dataframe,2,"value");
    BOOST_CHECK_EQUAL(element(),3);
  }
}
