/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#define BOOST_TEST_MODULE dataframe_element_proxy

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include "dataframe/element_proxy.hpp"
#include <boost/mpl/vector.hpp>

using test_types = boost::mpl::vector<int,long,long long,float,double,long double>;

BOOST_AUTO_TEST_CASE_TEMPLATE(test_typedef,T,test_types)
{
  using element_proxy_type = dataframe::element_proxy<T>;
  using const_element_proxy_type = dataframe::element_proxy<T const>;
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_default_constructor,T,test_types)
{
  using element_proxy_type = dataframe::element_proxy<T>;
  using const_element_proxy_type = dataframe::element_proxy<T const>;

  element_proxy_type element_proxy;
  const_element_proxy_type const_element_proxy;
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_element_proxy,T,test_types)
{
  using element_proxy_type = dataframe::element_proxy<T>;

  T t (42);
  element_proxy_type element_proxy(t);
  BOOST_CHECK_EQUAL(element_proxy,t);
  BOOST_CHECK_EQUAL(element_proxy/7,6);
  element_proxy = 13;
  BOOST_CHECK_EQUAL(element_proxy,13);
  element_proxy = T(17);
  BOOST_CHECK_EQUAL(element_proxy,17);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_const_element_proxy,T,test_types)
{
  using element_proxy_type = dataframe::element_proxy<T>;
  using const_element_proxy_type = dataframe::element_proxy<T const>;

  T t (42);
  const_element_proxy_type element_proxy(t);
  BOOST_CHECK_EQUAL(element_proxy,t);
  BOOST_CHECK_EQUAL(element_proxy/7,6);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_make_element_proxy,T,test_types)
{
  using element_proxy_type = dataframe::element_proxy<T>;
  using dataframe::make_element_proxy;

  T t (42);
  auto element_proxy = make_element_proxy<T>(t);
  static_assert(std::is_same<decltype(element_proxy),element_proxy_type>::value,"");
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_make_element_proxy_2,T,test_types)
{
  using element_proxy_type = dataframe::element_proxy<T,long double const>;
  using dataframe::make_element_proxy;

  T t (42);
  auto element_proxy = make_element_proxy<long double const>(t);
  static_assert(std::is_same<decltype(element_proxy),element_proxy_type>::value,"");
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_make_const_element_proxy,T,test_types)
{
  using const_element_proxy_type = dataframe::element_proxy<T const,T>;
  using dataframe::make_element_proxy;

  T const t (42);
  auto element_proxy = make_element_proxy<T>(t);
  static_assert(std::is_same<decltype(element_proxy),const_element_proxy_type>::value,"");
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_make_const_element_proxy_2,T,test_types)
{
  using const_element_proxy_type = dataframe::element_proxy<T const,long double>;
  using dataframe::make_element_proxy;

  T const t (42);
  auto element_proxy = make_element_proxy<long double>(t);
  static_assert(std::is_same<decltype(element_proxy),const_element_proxy_type>::value,"");
}
