/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#define BOOST_TEST_MODULE dataframe_missing_data

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <dataframe/missing_data.hpp>
#include <boost/mpl/vector.hpp>

BOOST_AUTO_TEST_CASE(test_missing_data)
{
  using dataframe::missing_data;
  {
    auto manipulator = missing_data(std::cout);
    BOOST_CHECK_EQUAL(manipulator.pattern(),"");
    BOOST_CHECK_EQUAL(manipulator.replacement(),"");
  }
  {
    std::cout << missing_data("?");
    auto manipulator = missing_data(std::cout);
    BOOST_CHECK_EQUAL(manipulator.pattern(),"?");
    BOOST_CHECK_EQUAL(manipulator.replacement(),"");
  }
  {
    std::cout << missing_data("NA","?");
    auto manipulator = missing_data(std::cout);
    BOOST_CHECK_EQUAL(manipulator.pattern(),"NA");
    BOOST_CHECK_EQUAL(manipulator.replacement(),"?");
  }
  {
    std::cout << "XXX - MANIPULATOR DATA ARE PERSISTENT" << std::endl;
    auto manipulator = missing_data(std::cout);
    BOOST_CHECK_EQUAL(manipulator.pattern(),"NA");
    BOOST_CHECK_EQUAL(manipulator.replacement(),"?");
  }
}

BOOST_AUTO_TEST_CASE(test_transform)
{
  using dataframe::missing_data;
  {
    auto manipulator = missing_data("NA","?");
    BOOST_CHECK_EQUAL(manipulator.transform(manipulator.pattern()),manipulator.replacement());
  }
  {
    auto manipulator = missing_data("^NA$","?");
    BOOST_CHECK_EQUAL(manipulator.transform("NA"),manipulator.replacement());
  }
  {
    auto manipulator = missing_data("^NA$","?");
    BOOST_CHECK_EQUAL(manipulator.transform("NA "),"NA ");
  }
}
