/*
 * Copyright (c) 2017 Gregory Penn.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#define BOOST_TEST_MODULE gdal

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <iostream>
#include <boost/mpl/vector.hpp>
#include <iostream>
// #include <gdal/raster_handle.hpp>
#include <gdal/grid_srs.hpp>


using std::cout;
using std::endl;
using namespace gdal;

#define rasterfile "../test_raster.tif"

BOOST_AUTO_TEST_CASE(grid_srs_default_constructor)
{
  grid_srs srs;
}

BOOST_AUTO_TEST_CASE(grid_srs_maxvalues_constructor)
{
  grid_srs srs(3,4);
}

BOOST_AUTO_TEST_CASE(grid_srs_copy_constructor)
{
  grid_srs srs(3,4);
  grid_srs srs_copy(srs);
}

BOOST_AUTO_TEST_CASE(grid_srs_move_constructor)
{
  grid_srs srs(3,4);

  auto srs_moved = std::move(srs);
}

BOOST_AUTO_TEST_CASE(grid_equality_comparable)
{
  int xmax1(3); int ymax1(4);
  int xmax2(5); int ymax2(6);
  grid_srs grid1(xmax1, ymax1);
  grid_srs grid1_copy(grid1);
  grid_srs grid2(xmax2, ymax1);
  grid_srs grid3(xmax1, ymax2);
  grid_srs grid4(xmax2, ymax2);

  BOOST_CHECK((grid1 == grid1_copy));
  BOOST_CHECK((grid1 != grid2));
  BOOST_CHECK((grid1 != grid3));
  BOOST_CHECK((grid1 != grid3));

  BOOST_CHECK_EQUAL(grid1, grid1_copy);
}
