/*
 * Copyright (c) 2017 Gregory Penn.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#define BOOST_TEST_MODULE gdal

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <iostream>
#include <gdal/vectordataset.hpp>
#include <gdal/vectorlayer.hpp>
#include <gdal/point.hpp>

using namespace gdal;

#define datasource "../data/tl_2010_35_county10/tl_2010_35_county10.shp"

BOOST_AUTO_TEST_CASE(vectorlayer_default_constructor)
{
  vectorlayer l;
}

BOOST_AUTO_TEST_CASE(vectorlayer_smartptr_constructor)
{
  smartptr<OGRLayer> layerp;
  vectorlayer layer(layerp);
}

BOOST_AUTO_TEST_CASE(vectorlayer_field_index)
{
  auto layer = vectordataset(datasource).layer(0);
  BOOST_CHECK_EQUAL(layer.field_index("NAME10"), 0);
}

BOOST_AUTO_TEST_CASE(vectorlayer_feature)
{
  auto layer = vectordataset(datasource).layer(0);
  layer.feature(1);
  BOOST_CHECK_EQUAL(layer.feature(1).FID(), 1);
}

BOOST_AUTO_TEST_CASE(vectorlayer_next_feature)
{
  auto dataset = vectordataset(datasource);
  auto layer = dataset.layer(0);
  layer.reset_reading();
  BOOST_CHECK_EQUAL(layer.next_feature().FID(), 0);
  BOOST_CHECK_EQUAL(layer.next_feature().FID(), 1);
}

BOOST_AUTO_TEST_CASE(vectorlayer_reset_reading)
{
  auto layer = vectordataset(datasource).layer(0);
  layer.next_feature();
  BOOST_CHECK_EQUAL(layer.next_feature().FID(), 1);
  layer.reset_reading();
  BOOST_CHECK_EQUAL(layer.next_feature().FID(), 0);
}

BOOST_AUTO_TEST_CASE(vectorlayer_field_value_as_string)
{
  auto layer = vectordataset(datasource).layer(0);
  auto value = layer.next_feature().field_as_string("NAME10");
  BOOST_CHECK_EQUAL(value, "Harding");
}

BOOST_AUTO_TEST_CASE(vectorlayer_spatial_filter)
{
  auto layer = vectordataset(datasource).layer(0);
  BOOST_ASSERT(!layer.spatial_filter());

  auto pt = point<geographic_srs>(-107.01599, 32.31111, make_wgs84());
  BOOST_CHECK_EQUAL(layer.spatial_filter(pt).next_feature().field_as_string("NAME10"), "Dona Ana");
}
