/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_MODULE graph_model_distribution_beta_beta_distribution

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <graph_model/distribution/beta/beta_distribution.hpp>
#include <graph_model/vertex/lit.hpp>

#include <graph_model/distribution/parameter/alpha.hpp>
#include <graph_model/distribution/parameter/beta.hpp>
#include <graph_model/distribution/parameter/precision.hpp>
#include <graph_model/distribution/parameter/probability.hpp>
#include <graph_model/distribution/parameter/variate.hpp>

#include <iostream>
#include <string>

#if XXX_TOO_MANY_ARGUMENTS
BOOST_AUTO_TEST_CASE(test_too_many_arguments)
{
  using graph_model::distribution::beta::beta_distribution;
  auto distribution = beta_distribution(1,2,3,4);
}
#endif

BOOST_AUTO_TEST_CASE(test_default_constructor)
{
  using graph_model::distribution::beta::beta_distribution;
  using graph_model::distribution::parameter::alpha;
  using graph_model::distribution::parameter::beta;
  using graph_model::distribution::parameter::variate;
  {
    auto distribution = beta_distribution();
    BOOST_CHECK_EQUAL(distribution.alpha(),1);
    BOOST_CHECK_EQUAL(distribution.beta(),1);
    BOOST_CHECK_EQUAL(distribution.probability(),0.5);
    BOOST_CHECK_EQUAL(distribution.precision(),1+1);
    BOOST_CHECK_EQUAL(distribution.variate(),0.5);
  }
  {
    auto distribution = beta_distribution(42);
    BOOST_CHECK_EQUAL(distribution.alpha(),42);
    BOOST_CHECK_EQUAL(distribution.beta(),1);
    BOOST_CHECK_EQUAL(distribution.probability(),42./(1+42));
    BOOST_CHECK_EQUAL(distribution.precision(),42+1);
    BOOST_CHECK_EQUAL(distribution.variate(),0.5);
  }
  {
    auto distribution = beta_distribution(42,13);
    BOOST_CHECK_EQUAL(distribution.alpha(),42);
    BOOST_CHECK_EQUAL(distribution.beta(),13);
    BOOST_CHECK_EQUAL(distribution.probability(),42./(42+13));
    BOOST_CHECK_EQUAL(distribution.precision(),42+13);
    BOOST_CHECK_EQUAL(distribution.variate(),0.5);
  }
  {
    auto distribution = beta_distribution(42,13,17);
    BOOST_CHECK_EQUAL(distribution.alpha(),42);
    BOOST_CHECK_EQUAL(distribution.beta(),13);
    BOOST_CHECK_EQUAL(distribution.probability(),42./(42+13));
    BOOST_CHECK_EQUAL(distribution.precision(),42+13);
    BOOST_CHECK_EQUAL(distribution.variate(),17);
  }
  {
    auto distribution = beta_distribution(alpha=42);
    BOOST_CHECK_EQUAL(distribution.alpha(),42);
    BOOST_CHECK_EQUAL(distribution.beta(),1);
    BOOST_CHECK_EQUAL(distribution.probability(),42./(42+1));
    BOOST_CHECK_EQUAL(distribution.precision(),42+1);
    BOOST_CHECK_EQUAL(distribution.variate(),0.5);
  }
  {
    auto distribution = beta_distribution(alpha=42,13);
    BOOST_CHECK_EQUAL(distribution.alpha(),42);
    BOOST_CHECK_EQUAL(distribution.beta(),13);
    BOOST_CHECK_EQUAL(distribution.probability(),42./(42+13));
    BOOST_CHECK_EQUAL(distribution.precision(),42+13);
    BOOST_CHECK_EQUAL(distribution.variate(),0.5);
  }
  {
    auto distribution = beta_distribution(13,alpha=42);
    BOOST_CHECK_EQUAL(distribution.alpha(),42);
    BOOST_CHECK_EQUAL(distribution.beta(),13);
    BOOST_CHECK_EQUAL(distribution.probability(),42./(42+13));
    BOOST_CHECK_EQUAL(distribution.precision(),42+13);
    BOOST_CHECK_EQUAL(distribution.variate(),0.5);
  }
  {
    auto distribution = beta_distribution(beta=2);
    BOOST_CHECK_EQUAL(distribution.alpha(),1);
    BOOST_CHECK_EQUAL(distribution.beta(),2);
    BOOST_CHECK_EQUAL(distribution.probability(),1./(1+2));
    BOOST_CHECK_EQUAL(distribution.precision(),1+2);
    BOOST_CHECK_EQUAL(distribution.variate(),0.5);
  }
  {
    auto distribution = beta_distribution(alpha=1,beta=2);
    BOOST_CHECK_EQUAL(distribution.alpha(),1);
    BOOST_CHECK_EQUAL(distribution.beta(),2);
    BOOST_CHECK_EQUAL(distribution.probability(),1./(1+2));
    BOOST_CHECK_EQUAL(distribution.precision(),1+2);
    BOOST_CHECK_EQUAL(distribution.variate(),0.5);
  }
  {
    auto distribution = beta_distribution(alpha=2,beta=3,variate=4);
    BOOST_CHECK_EQUAL(distribution.alpha(),2);
    BOOST_CHECK_EQUAL(distribution.beta(),3);
    BOOST_CHECK_EQUAL(distribution.probability(),2./(2+3));
    BOOST_CHECK_EQUAL(distribution.precision(),2+3);
    BOOST_CHECK_EQUAL(distribution.variate(),4);
  }
}

BOOST_AUTO_TEST_CASE(test_constructor)
{
  using graph_model::distribution::beta::beta_distribution;
  {
    auto distribution = beta_distribution();
    BOOST_CHECK_EQUAL(distribution.alpha(),1);
    BOOST_CHECK_EQUAL(distribution.beta(),1);
    BOOST_CHECK_EQUAL(distribution.precision(),1+1);
    BOOST_CHECK_EQUAL(distribution.variate(),0.5);
  }
  {
    auto distribution = beta_distribution(0);
    BOOST_CHECK_EQUAL(distribution.alpha(),0);
    BOOST_CHECK_EQUAL(distribution.beta(),1);
    BOOST_CHECK_EQUAL(distribution.precision(),0+1);
    BOOST_CHECK_EQUAL(distribution.variate(),0.5);
  }
  {
    auto distribution = beta_distribution(1,2);
    BOOST_CHECK_EQUAL(distribution.alpha(),1);
    BOOST_CHECK_EQUAL(distribution.beta(),2);
    BOOST_CHECK_EQUAL(distribution.precision(),1+2);
    BOOST_CHECK_EQUAL(distribution.variate(),0.5);
  }
  {
    auto distribution = beta_distribution(2,3,4);
    BOOST_CHECK_EQUAL(distribution.alpha(),2);
    BOOST_CHECK_EQUAL(distribution.beta(),3);
    BOOST_CHECK_EQUAL(distribution.variate(),4);
    BOOST_CHECK_EQUAL(distribution.precision(),2+3);
  }
}

BOOST_AUTO_TEST_CASE(test_constructor_lit)
{
  using graph_model::distribution::beta::beta_distribution;
  using graph_model::vertex::lit;

  auto alpha = lit(42);
  auto distribution = beta_distribution(alpha,1,2);
  BOOST_CHECK_EQUAL(distribution.alpha(),42);
  BOOST_CHECK_EQUAL(distribution.beta(),1);
  BOOST_CHECK_EQUAL(distribution.precision(),42+1);
  BOOST_CHECK_EQUAL(distribution.variate(),2);
}

BOOST_AUTO_TEST_CASE(test_constructor_expr)
{
  using graph_model::distribution::beta::beta_distribution;
  using graph_model::vertex::lit;

  auto variate = 42;
  auto alpha = lit(0) + 7;
  auto beta = 13 * lit(3);
  auto distribution = beta_distribution(alpha,beta,variate);
  BOOST_CHECK_EQUAL(distribution.alpha(),0 + 7);
  BOOST_CHECK_EQUAL(distribution.beta(),13 * 3);
  BOOST_CHECK_EQUAL(distribution.precision(),(0 + 7) + (13 * 3));
  BOOST_CHECK_EQUAL(distribution.variate(),42);
}

BOOST_AUTO_TEST_CASE(test_constructor_tagged_parameters)
{
  using std::sqrt;

  using graph_model::distribution::beta::beta_distribution;
  using graph_model::distribution::parameter::alpha;
  using graph_model::distribution::parameter::beta;
  using graph_model::distribution::parameter::precision;
  using graph_model::distribution::parameter::probability;
  using graph_model::distribution::parameter::variate;
  using graph_model::vertex::lit;

  auto v = 42;
  auto a = lit(0) + 7;
  auto b = 13 * lit(3);
  auto p = 0.25;
  auto n = 5;

  {
    auto distribution
      = beta_distribution(variate=v,alpha=a,beta=b);
    BOOST_CHECK_EQUAL(distribution.alpha(),0 + 7);
    BOOST_CHECK_EQUAL(distribution.beta(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.precision(),(0 + 7) + (13 * 3));
    BOOST_CHECK_EQUAL(distribution.variate(),42);
  }
  {
    auto distribution
      = beta_distribution(probability=p);
    BOOST_CHECK_EQUAL(distribution.alpha(),p * 2);
    BOOST_CHECK_EQUAL(distribution.beta(),(1 - p) * 2);
    BOOST_CHECK_EQUAL(distribution.precision(),2);
    BOOST_CHECK_EQUAL(distribution.variate(),0.5);
  }
  {
    auto distribution
      = beta_distribution(variate=v,probability=p,precision=n);
    BOOST_CHECK_EQUAL(distribution.alpha(),p * n);
    BOOST_CHECK_EQUAL(distribution.beta(),(1 - p) * n);
    BOOST_CHECK_EQUAL(distribution.precision(),n);
    BOOST_CHECK_EQUAL(distribution.variate(),v);
  }
}

BOOST_AUTO_TEST_CASE(test_constructor_tagged_parameter_pack)
{
  using graph_model::distribution::beta::beta_distribution;
  using graph_model::distribution::parameter::alpha;
  using graph_model::distribution::parameter::beta;
  using graph_model::distribution::parameter::precision;
  using graph_model::distribution::parameter::probability;
  using graph_model::distribution::parameter::variate;
  using graph_model::vertex::lit;

  auto v = 42;
  auto a = lit(0) + 7;
  auto b = 13 * lit(3);
  auto p = 0.25;
  auto n = 5;

  {
    auto distribution
      = beta_distribution(variate=v,alpha=a,beta=b);
    BOOST_CHECK_EQUAL(distribution.alpha(),0 + 7);
    BOOST_CHECK_EQUAL(distribution.beta(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.precision(),(0 + 7) + (13 * 3));
    BOOST_CHECK_EQUAL(distribution.variate(),42);
  }
  {
    auto distribution
      = beta_distribution(variate=v,alpha=a,beta=b);
    BOOST_CHECK_EQUAL(distribution.alpha(),0 + 7);
    BOOST_CHECK_EQUAL(distribution.beta(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.precision(),(0 + 7) + (13 * 3));
    BOOST_CHECK_EQUAL(distribution.variate(),42);
  }
  {
    auto distribution
      = beta_distribution(variate=v,alpha=a,beta=b);
    BOOST_CHECK_EQUAL(distribution.alpha(),0 + 7);
    BOOST_CHECK_EQUAL(distribution.beta(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.precision(),(0 + 7) + (13 * 3));
    BOOST_CHECK_EQUAL(distribution.variate(),42);
  }
  {
    auto distribution
      = beta_distribution(variate=v,a,b);
    BOOST_CHECK_EQUAL(distribution.alpha(),0 + 7);
    BOOST_CHECK_EQUAL(distribution.beta(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.precision(),(0 + 7) + (13 * 3));
    BOOST_CHECK_EQUAL(distribution.variate(),42);
  }
  {
    auto distribution
      = beta_distribution(b,alpha=a,v);
    BOOST_CHECK_EQUAL(distribution.alpha(),0 + 7);
    BOOST_CHECK_EQUAL(distribution.beta(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.precision(),(0 + 7) + (13 * 3));
    BOOST_CHECK_EQUAL(distribution.variate(),42);
  }
  {
    auto distribution
      = beta_distribution(beta=b,a,v);
    BOOST_CHECK_EQUAL(distribution.alpha(),0 + 7);
    BOOST_CHECK_EQUAL(distribution.beta(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.precision(),(0 + 7) + (13 * 3));
    BOOST_CHECK_EQUAL(distribution.variate(),42);
  }
  {
    auto distribution
      = beta_distribution(b,alpha=a,42);
    BOOST_CHECK_EQUAL(distribution.alpha(),0 + 7);
    BOOST_CHECK_EQUAL(distribution.beta(),13 * 3);
    BOOST_CHECK_EQUAL(distribution.precision(),(0 + 7) + (13 * 3));
    BOOST_CHECK_EQUAL(distribution.variate(),42);
  }
}
