/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_MODULE graph_model_distribution_dirichlet_dirichlet_distribution

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <graph_model/distribution/dirichlet/dirichlet_distribution.hpp>

#include <graph_model/distribution/parameter/alpha.hpp>
#include <graph_model/distribution/parameter/precision.hpp>
#include <graph_model/distribution/parameter/mean.hpp>
#include <graph_model/distribution/parameter/variate.hpp>

#include <sample_space/vector/simplex.hpp>
#include <sample_space/vector/vector.hpp>

BOOST_AUTO_TEST_CASE(test_constructor)
{
  using simplex_type = sample_space::vector::simplex<double>;
  using vector_type = sample_space::vector::vector<double>;

  using graph_model::distribution::dirichlet::dirichlet_distribution;

  namespace parameter = graph_model::distribution::parameter;

  simplex_type variate (3);
  variate.at(2) = 0.1;
  variate.at(1) = 0.2;
  variate.at(0) = 0.7;

  vector_type alpha (3);
  alpha.at(2) = 1;
  alpha.at(1) = 1;
  alpha.at(0) = 1;

  { auto distribution = dirichlet_distribution(); }
  { auto distribution = dirichlet_distribution(alpha); }
  { auto distribution = dirichlet_distribution(alpha,variate); }
  { auto distribution = dirichlet_distribution(parameter::alpha=alpha); }
  { auto distribution = dirichlet_distribution(parameter::alpha=alpha,variate); }
  { auto distribution = dirichlet_distribution(variate,parameter::alpha=alpha); }
  { auto distribution = dirichlet_distribution(parameter::alpha=alpha); }
  { auto distribution = dirichlet_distribution(parameter::alpha=alpha,parameter::variate=variate); }
  { auto distribution = dirichlet_distribution(parameter::mean=alpha,parameter::precision=10,parameter::variate=variate); }
}
