/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_MODULE graph_model_distribution_dirichlet_dirichlet_strategy

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <graph_model/distribution/dirichlet/dirichlet_strategy.hpp>
#include <sample_space/vector/simplex.hpp>
#include <sample_space/vector/vector.hpp>

class parameter_pack {};

BOOST_AUTO_TEST_CASE(test_default_constructor)
{
  using tag = graph_model::distribution::dirichlet::tag::alpha;
  using dirichlet_strategy_type
    = graph_model::distribution::dirichlet::dirichlet_strategy<tag,parameter_pack>;
  dirichlet_strategy_type strategy;
}

BOOST_AUTO_TEST_CASE(test_constructor_function)
{
  using simplex_type = sample_space::vector::simplex<double>;
  using vector_type = sample_space::vector::vector<double>;

  using graph_model::distribution::dirichlet::make_dirichlet_strategy;
  using graph_model::distribution::parameter::make_parameter;
  using graph_model::distribution::parameter::make_parameter_pack;

  namespace dirichlet = graph_model::distribution::dirichlet;
  namespace tag = graph_model::distribution::parameter::tag;

  simplex_type variate (3);
  variate.at(2) = 0.1;
  variate.at(1) = 0.2;
  variate.at(0) = 0.7;

  vector_type alpha (3);
  alpha.at(2) = 1;
  alpha.at(1) = 1;
  alpha.at(0) = 1;

  auto strategy = make_dirichlet_strategy<dirichlet::tag::alpha>
    (make_parameter_pack
     (make_parameter<tag::alpha>().default_value(alpha),
      make_parameter<tag::variate>().default_value(variate)));
  strategy.sample();
  strategy.pdf();
}
