/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_MODULE graph_model_distribution_parameter_parameter_pack

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <graph_model/distribution/parameter/parameter_pack.hpp>

class tag1 {};
class tag2 {};

template < int N >
struct f
{
  auto operator () () const { return N; }
};

BOOST_AUTO_TEST_CASE(test_defaults)
{
  using graph_model::distribution::parameter::make_parameter_pack;
  using graph_model::distribution::parameter::make_parameter;
  auto parameter_pack
    = make_parameter_pack(make_parameter<tag1>().default_value(13),make_parameter<tag2>().default_value(42));
  BOOST_CHECK_EQUAL(parameter_pack.value<tag1>(),13);
  BOOST_CHECK_EQUAL(parameter_pack.value<tag2>(),42);
}

BOOST_AUTO_TEST_CASE(test_defaults_2)
{
  using graph_model::distribution::parameter::make_parameter_pack;
  using graph_model::distribution::parameter::make_parameter;
  auto parameter_pack
    = make_parameter_pack(make_parameter<tag2>().default_value(42),make_parameter<tag1>().default_value(13));
  BOOST_CHECK_EQUAL(parameter_pack.value<tag1>(),13);
  BOOST_CHECK_EQUAL(parameter_pack.value<tag2>(),42);
}

BOOST_AUTO_TEST_CASE(test_add_1)
{
  using graph_model::distribution::parameter::make_parameter_pack;
  using graph_model::distribution::parameter::make_parameter;
  using graph_model::vertex::make_expression_function;
  auto parameter_pack
    = make_parameter_pack(make_parameter<tag1>(),make_parameter<tag2>().default_value(42))
    .add<tag1>(make_expression_function(13));
  BOOST_CHECK_EQUAL(parameter_pack.value<tag1>(),13);
  BOOST_CHECK_EQUAL(parameter_pack.value<tag2>(),42);
}

BOOST_AUTO_TEST_CASE(test_add_2)
{
  using graph_model::distribution::parameter::make_parameter_pack;
  using graph_model::distribution::parameter::make_parameter;
  using graph_model::vertex::make_expression_function;
  auto parameter_pack
    = make_parameter_pack(make_parameter<tag1>().default_value(13),make_parameter<tag2>())
    .add<tag2>(make_expression_function(42));
  BOOST_CHECK_EQUAL(parameter_pack.value<tag1>(),13);
  BOOST_CHECK_EQUAL(parameter_pack.value<tag2>(),42);
}

BOOST_AUTO_TEST_CASE(test_add_1_2)
{
  using graph_model::distribution::parameter::make_parameter_pack;
  using graph_model::distribution::parameter::make_parameter;
  using graph_model::vertex::make_expression_function;
  auto parameter_pack
    = make_parameter_pack(make_parameter<tag1>(),make_parameter<tag2>())
    .add<tag1>(make_expression_function(13))
    .add<tag2>(make_expression_function(42));
  BOOST_CHECK_EQUAL(parameter_pack.value<tag1>(),13);
  BOOST_CHECK_EQUAL(parameter_pack.value<tag2>(),42);
}

BOOST_AUTO_TEST_CASE(test_add_2_1)
{
  using graph_model::distribution::parameter::make_parameter_pack;
  using graph_model::distribution::parameter::make_parameter;
  using graph_model::vertex::make_expression_function;
  auto parameter_pack
    = make_parameter_pack(make_parameter<tag1>(),make_parameter<tag2>())
    .add<tag2>(make_expression_function(42))
    .add<tag1>(make_expression_function(13));
  BOOST_CHECK_EQUAL(parameter_pack.value<tag1>(),13);
  BOOST_CHECK_EQUAL(parameter_pack.value<tag2>(),42);
}
