/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_MODULE graph_model_distribution_uniform_uniform_strategy

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <graph_model/distribution/uniform/uniform_strategy.hpp>

class parameter_pack {};

#include <boost/mpl/vector.hpp>

using test_types = boost::mpl::vector< float,double,long double >;

BOOST_AUTO_TEST_CASE(test_default_constructor)
{
  using uniform_strategy_type
    = graph_model::distribution::uniform::uniform_strategy<parameter_pack>;
  uniform_strategy_type strategy;
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_constructor_function,T,test_types)
{
  using lower_bound = graph_model::distribution::parameter::tag::lower_bound;
  using upper_bound = graph_model::distribution::parameter::tag::upper_bound;
  using variate = graph_model::distribution::parameter::tag::variate;

  using boost::hana::make_map;
  using boost::hana::make_pair;
  using boost::hana::type_c;
  using graph_model::distribution::uniform::make_uniform_strategy;
  using graph_model::distribution::parameter::make_parameter;
  using graph_model::distribution::parameter::make_parameter_pack;
  using graph_model::vertex::make_expression_function;

  auto strategy = make_uniform_strategy
    (make_parameter_pack
     (make_parameter<lower_bound>().default_value(-2),
      make_parameter<upper_bound>().default_value(+2),
      make_parameter<variate>().default_value(T(1))));
  strategy.sample();
  BOOST_CHECK_EQUAL(strategy.pdf(),0.25);
}
