#
# Copyright (c) 2017 Brook Milligan.
#
# This file is part of the GraphModel Library.
#
#     The GraphModel Library is free software: you can redistribute it
#     and/or modify it under the terms of the GNU General Public
#     License as published by the Free Software Foundation, either
#     version 3 of the License, or (at your option) any later version.
#
#     The GraphModel Library is distributed in the hope that it will
#     be useful, but WITHOUT ANY WARRANTY; without even the implied
#     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#     See the GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public
#     License along with the GraphModel Library.  If not, see
#     <http://www.gnu.org/licenses/>.
#

library(ggplot2)
library(dplyr)
library(tidyr)

logistic <- function (x) { 1/(1 + exp(-x)) }
linear_predictor <- function (b0, b1, x) { b0 + b1*x }
bernoulli <- function (p) { rbinom(length(p),1,p) }
model_y <- function (x) { linear_predictor(b0, b1, x) %>% logistic() %>% bernoulli() }

label <- expression(p)

mcmc.filename <- 'mcmc.dat'
pdfname <- 'mcmc.pdf'
burnin <- 500

samples.filename <- 'samples.dat'
samples.df <- read.delim(samples.filename)
model <- glm(observed ~ x, family = binomial(link='logit'), data = samples.df)
b0_est <- coef(model)[[1]]
b1_est <- coef(model)[[2]]

mcmc.df <- read.delim(mcmc.filename) %>%
  filter(i > burnin) %>%
  group_by(var)

summary.df <- summarize(mcmc.df, mean = mean(MCMC_sample))
b0_est <- filter(summary.df, var == "b0")$mean
b1_est <- filter(summary.df, var == "b1")$mean


pdf(pdfname,width=7,height=5)
ggplot(mcmc.df, aes(x = MCMC_sample)) +
  facet_wrap(~var, nrow = 1, scales = "free") +
  geom_histogram(color='gray',fill='gray') +
  geom_vline(data = summary.df, aes(xintercept = mean)) +
  labs(x='Value',y='Count') +
  theme_bw()

ggplot(mcmc.df, aes(x = i, y = MCMC_sample)) +
  theme_bw() +
  labs(x='MCMC iteration',y='Value') +
  geom_step(color='gray') +
  geom_hline(aes(yintercept = mean), data = summary.df) +
  facet_wrap(~var, ncol = 1, scales = "free_y")

ggplot(samples.df, aes(x=x, y=observed)) +
  stat_function(fun = function (x) { logistic(linear_predictor(b0_est, b1_est, x)) }, color = "blue") +
  geom_point(data = samples.df, aes(y=observed)) +
  # geom_segment(aes(xend=x, yend=p), alpha=0.2) +
  geom_smooth(method = "glm", method.args = list(family = binomial(link='logit')),
              se = FALSE, color = "grey", alpha = 0.4) +
  ylab("p")

dev.off()
