#
# Copyright (c) 2017 Brook Milligan.
#
# This file is part of the GraphModel Library.
#
#     The GraphModel Library is free software: you can redistribute it
#     and/or modify it under the terms of the GNU General Public
#     License as published by the Free Software Foundation, either
#     version 3 of the License, or (at your option) any later version.
#
#     The GraphModel Library is distributed in the hope that it will
#     be useful, but WITHOUT ANY WARRANTY; without even the implied
#     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#     See the GNU General Public License for more details.
#
#     You should have received a copy of the GNU General Public
#     License along with the GraphModel Library.  If not, see
#     <http://www.gnu.org/licenses/>.
#

library(bit64)
library(data.table)
library(ggplot2)

title <- 'Diagnostics'

datname <- 'diagnostics.dat'
pdfname.histograms <- 'diagnostics-histograms.pdf'
pdfname.tasks <- 'diagnostics-tasks.pdf'

data <- fread(datname)
first.time <- min(data$posted,data$started,data$finished)
data[ , posted := (data$posted - first.time) / 1000 ]
data[ , started := (data$started - first.time) / 1000 ]
data[ , finished := (data$finished - first.time) / 1000 ]
setkey(data,posted)
data[ , id := as.factor(1:nrow(data)) ]
data[ is.na(type), type := '' ]
data[ , type := as.factor(type) ]
data$Task <- paste(data$name,data$type,sep=': ')
data$Task <- as.factor(data$Task)
data[ , short.label := Task ]
data[ , short.label := gsub('scheduler','s',short.label) ]
data[ , short.label := gsub('threadpool','t',short.label) ]
data[ , short.label := gsub('graph_model::','',short.label) ]
data[ , short.label := gsub('signals::','',short.label) ]
data[ , short.label := gsub('tag::','',short.label) ]
str(data)
summary(data)
data

print('data.subset')
min.time <- 0
n.events <- 5000
include.rows <- data$posted >= min.time
data.subset <- data[include.rows,]
data.subset <- data.subset[1:min(n.events,nrow(data.subset)),]
str(data.subset)
summary(data.subset)
data.subset

pdf(pdfname.histograms,width=7,height=40)
ggplot(data,aes(x=waited)) +
  theme_bw() +
  labs(title=title,x=expression(Waiting~time~(mu * s)),y='Count') +
  geom_histogram(color='gray',fill='gray') +
  facet_grid(short.label ~ .)
ggplot(data,aes(x=lasted)) +
  theme_bw() +
  labs(title=title,x=expression(Execution~time~(mu * s)),y='Count') +
  geom_histogram(color='gray',fill='gray') +
  facet_grid(short.label ~ .)
dev.off()

pdf(pdfname.tasks,width=36,height=20)
ggplot(data.subset,aes(y=id,yend=id,group=id,color=Task)) +
  theme_bw() +
  labs(title=title,x=expression(Running~time~(ms)),y='Job') +
  geom_segment(aes(x=posted,xend=started),linetype='dotted') +
  geom_segment(aes(x=started,xend=finished),linetype='solid',size=3)
dev.off()

q()

ggplot(data,aes(y=id,yend=id,group=id)) +
  theme(plot.background=element_blank(),
        axis.line=element_line(color='black'),
        axis.text.y=element_blank(),
        axis.ticks.y=element_blank()) +
  labs(title=title,x=expression(Running~time~(ms)),y='Job') +
  geom_segment(aes(x=posted,xend=started),linetype='dotted') +
  geom_segment(aes(x=started,xend=finished),linetype='solid')
