/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_MODULE graph_model_vertex_expression

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <graph_model/concept/expression_archetype.hpp>
#include <graph_model/concept/expression_concept.hpp>
#include <graph_model/concept/expression_policy_archetype.hpp>
#include <graph_model/vertex/expression.hpp>

#include <boost/mpl/vector.hpp>

template < typename T >
struct expression_policy
{
  template < typename ... U >
  auto operator () (U&& ...) const { return T{42}; }
};

using test_types = boost::mpl::vector<int,long,float,double,long double>;

BOOST_AUTO_TEST_CASE(test_expression_archetype)
{
  using expression_type = graph_model::concept::expression_archetype<>;
  BOOST_CONCEPT_ASSERT((graph_model::concept::expression_concept<expression_type>));
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_default_constructor,T,test_types)
{
  using expression_type = graph_model::vertex::expression<T>;
  expression_type expr = {};
#if XXX_UNDEFINED_BEHAVIOR
  expr();
  expr.eval();
#endif	// XXX_UNDEFINED_BEHAVIOR
  expression_type other = std::move(expr);
}

#if XXX_IMPLICITLY_DELETED_COPY_CONSTRUCTOR
BOOST_AUTO_TEST_CASE_TEMPLATE(test_copy_constructor,T,test_types)
{
  using expression_type = graph_model::vertex::expression<T>;
  expression_type expr = {};
  expression_type other { expr };
}
#endif	// XXX_IMPLICITLY_DELETED_COPY_CONSTRUCTOR

BOOST_AUTO_TEST_CASE_TEMPLATE(test_move_constructor,T,test_types)
{
  using expression_type = graph_model::vertex::expression<T>;
  expression_type expr = {};
  expression_type other = std::move(expr);
}

#if XXX_IMPLICITLY_DELETED_COPY_ASSIGNMENT
BOOST_AUTO_TEST_CASE_TEMPLATE(test_copy_assignment,T,test_types)
{
  using expression_type = graph_model::vertex::expression<T>;
  expression_type expr = {};
  expression_type other = {};
  other = expr;
}
#endif	// XXX_IMPLICITLY_DELETED_COPY_ASSIGNMENT

BOOST_AUTO_TEST_CASE_TEMPLATE(test_move_assignment,T,test_types)
{
  using expression_type = graph_model::vertex::expression<T>;
  expression_type expr = {};
  expression_type other = {};
  other = std::move(expr);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_reset_strategy,T,test_types)
{
  using expression_type = graph_model::vertex::expression<T>;
  expression_type expr = {};
  expr.reset_strategy(graph_model::concept::expression_policy_archetype<T>());
#if XXX_UNDEFINED_BEHAVIOR
  expr();
  expr.eval();
#endif	// XXX_UNDEFINED_BEHAVIOR
  expr.reset_strategy(expression_policy<T>());
  BOOST_CHECK_EQUAL(expr(),42);
  BOOST_CHECK_EQUAL(expr.eval(),42);
}
