/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#define BOOST_TEST_MODULE graph_model_vertex_log

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <graph_model/vertex/log.hpp>
#include <graph_model/vertex/lit.hpp>

#include <boost/mpl/vector.hpp>

using test_types = boost::mpl::vector<int,long,float,double,long double>;

BOOST_AUTO_TEST_CASE_TEMPLATE(test_log,T,test_types)
{
  using graph_model::vertex::log;

  auto f = log(T(42));
  BOOST_CHECK_EQUAL(f(),std::log(T(42)));
  BOOST_CHECK_EQUAL(log(T(42))(),std::log(T(42)));
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_evaluate,T,test_types)
{
  using graph_model::vertex::lit;
  using graph_model::vertex::log;
  using boost::yap::evaluate;

  auto f = log(lit(T(42))) + lit(3);
  BOOST_CHECK_EQUAL(evaluate(f),std::log(T(42))+3);
  BOOST_CHECK_EQUAL(evaluate(log(T(42))+lit(3)),std::log(T(42))+3);
}
