/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_MODULE graph_model_vertex_scalar_factor

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <graph_model/concept/scalar_factor_archetype.hpp>
#include <graph_model/concept/scalar_factor_concept.hpp>
#include <graph_model/concept/scalar_factor_policy_archetype.hpp>
#include <graph_model/vertex/scalar_factor.hpp>

#include <boost/mpl/vector.hpp>

template < typename T >
struct scalar_factor_policy
{
  template < typename ... U >
  auto pdf (U&& ...) const { return T{0.25}; }
  template < typename ... U >
  auto cdf (U&& ...) const { return T{0.125}; }
};

using test_types = boost::mpl::vector<float,double,long double>;

BOOST_AUTO_TEST_CASE(test_scalar_factor_archetype)
{
  using scalar_factor_type = graph_model::concept::scalar_factor_archetype<>;
  BOOST_CONCEPT_ASSERT((graph_model::concept::scalar_factor_concept<scalar_factor_type>));
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_default_constructor,T,test_types)
{
  using scalar_factor_type = graph_model::vertex::scalar_factor<T,T>;
  scalar_factor_type scalar_factor = {};
#if XXX_UNDEFINED_BEHAVIOR
  scalar_factor();
  scalar_factor.eval();
  scalar_factor.pdf();
  scalar_factor.cdf();
#endif	// XXX_UNDEFINED_BEHAVIOR
  scalar_factor_type other = std::move(scalar_factor);
}

#if XXX_IMPLICITLY_DELETED_COPY_CONSTRUCTOR
BOOST_AUTO_TEST_CASE_TEMPLATE(test_copy_constructor,T,test_types)
{
  using scalar_factor_type = graph_model::vertex::scalar_factor<T>;
  scalar_factor_type scalar_factor = {};
  scalar_factor_type other { scalar_factor };
}
#endif	// XXX_IMPLICITLY_DELETED_COPY_CONSTRUCTOR

BOOST_AUTO_TEST_CASE_TEMPLATE(test_move_constructor,T,test_types)
{
  using scalar_factor_type = graph_model::vertex::scalar_factor<T,T>;
  scalar_factor_type scalar_factor = {};
  scalar_factor_type other = std::move(scalar_factor);
}

#if XXX_IMPLICITLY_DELETED_COPY_ASSIGNMENT
BOOST_AUTO_TEST_CASE_TEMPLATE(test_copy_assignment,T,test_types)
{
  using scalar_factor_type = graph_model::vertex::scalar_factor<T,T>;
  scalar_factor_type scalar_factor = {};
  scalar_factor_type other = {};
  other = scalar_factor;
}
#endif	// XXX_IMPLICITLY_DELETED_COPY_ASSIGNMENT

BOOST_AUTO_TEST_CASE_TEMPLATE(test_move_assignment,T,test_types)
{
  using scalar_factor_type = graph_model::vertex::scalar_factor<T,T>;
  scalar_factor_type scalar_factor = {};
  scalar_factor_type other = {};
  other = std::move(scalar_factor);
}

BOOST_AUTO_TEST_CASE_TEMPLATE(test_reset_strategy,T,test_types)
{
  using scalar_factor_type = graph_model::vertex::scalar_factor<T,T>;
  scalar_factor_type scalar_factor = {};
  scalar_factor.reset_strategy(graph_model::concept::scalar_factor_policy_archetype<T,T>());
#if XXX_UNDEFINED_BEHAVIOR
  scalar_factor();
  scalar_factor.eval();
  scalar_factor.pdf();
  scalar_factor.cdf();
#endif	// XXX_UNDEFINED_BEHAVIOR
  scalar_factor.reset_strategy(scalar_factor_policy<T>());
  BOOST_CHECK_EQUAL(scalar_factor(),0.25);
  BOOST_CHECK_EQUAL(scalar_factor.eval(),0.25);
  BOOST_CHECK_EQUAL(scalar_factor.pdf(),0.25);
  BOOST_CHECK_EQUAL(scalar_factor.cdf(),0.125);
}
