/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_MODULE logging_initialize

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

#include <logging/global_log.hpp>
#include <logging/initialize.hpp>
#include <logging/logging_severity_level.hpp>
#include <logging/logging_status.hpp>
#include <boost/log/expressions.hpp>
#include <boost/log/sources/severity_logger.hpp>
#include <boost/log/sources/severity_channel_logger.hpp>
#include <boost/log/trivial.hpp>

BOOST_LOG_ATTRIBUTE_KEYWORD(channel,"Channel",std::string)

BOOST_AUTO_TEST_CASE(test_initialize)
{
  using severity_level = boost::log::trivial::severity_level;
  using log_type = boost::log::sources::severity_logger<severity_level>;

  using namespace boost::log::trivial;

  logging::enable();

  std::cout << "logging_severity_level: " << logging::logging_severity_level::apply() << std::endl;

  logging::initialize();

  log_type log;

  GLOBAL_LOG(log, trace) << "A trace severity message: " << trace;
  GLOBAL_LOG(log, debug) << "A debug severity message: " << debug;
  GLOBAL_LOG(log, info) << "An informational severity message: " << info;
  GLOBAL_LOG(log, warning) << "A warning severity message: " << warning;
  GLOBAL_LOG(log, error) << "An error severity message: " << error;
  GLOBAL_LOG(log, fatal) << "A fatal severity message: " << fatal;
}

BOOST_AUTO_TEST_CASE(test_initialize_file)
{
  using severity_level = boost::log::trivial::severity_level;
  using log_type = boost::log::sources::severity_channel_logger<severity_level>;

  using namespace boost::log::trivial;

  logging::enable();

  std::cout << "logging_severity_level: " << logging::logging_severity_level::apply() << std::endl;

  logging::initialize("test.log");

  log_type log;

  GLOBAL_LOG(log, trace) << "A trace severity message: " << trace;
  GLOBAL_LOG(log, debug) << "A debug severity message: " << debug;
  GLOBAL_LOG(log, info) << "An informational severity message: " << info;
  GLOBAL_LOG(log, warning) << "A warning severity message: " << warning;
  GLOBAL_LOG(log, error) << "An error severity message: " << error;
  GLOBAL_LOG(log, fatal) << "A fatal severity message: " << fatal;
}

BOOST_AUTO_TEST_CASE(test_initialize_filter)
{

  using severity_level = boost::log::trivial::severity_level;
  using log_type = boost::log::sources::severity_channel_logger<severity_level>;

  using namespace boost::log::trivial;
  namespace expr = boost::log::expressions;

  logging::enable();

  logging::initialize("filter-channel.log",
		      expr::begins_with(expr::attr<std::string>("Channel"),"chan")
		      && severity >= boost::log::trivial::info);
  logging::initialize("filter-XXX.log",channel == "XXX");

  log_type log (boost::log::keywords::channel="channel");

  GLOBAL_LOG(log, trace) << "A trace severity message: " << trace;
  GLOBAL_LOG(log, debug) << "A debug severity message: " << debug;
  GLOBAL_LOG(log, info) << "An informational severity message: " << info;
  GLOBAL_LOG(log, warning) << "A warning severity message: " << warning;
  GLOBAL_LOG(log, error) << "An error severity message: " << error;
  GLOBAL_LOG(log, fatal) << "A fatal severity message: " << fatal;
}

