/*
 * Copyright (c) 2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

#define BOOST_TEST_MODULE logging_initialize_trace_log

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

#include <logging/initialize_trace_log.hpp>
#include <logging/logging_status.hpp>
#include <string>
#include <boost/log/sources/severity_logger.hpp>
#include <boost/log/trivial.hpp>

BOOST_AUTO_TEST_CASE(test_initialize_trace_log)
{
  using severity_level = boost::log::trivial::severity_level;
  using log_type = boost::log::sources::severity_logger<severity_level>;

  using namespace boost::log::trivial;
  namespace keywords = boost::log::keywords;

  logging::enable();

  logging::initialize_trace_log<severity_level>::apply
    (keywords::file_name="test.log",
     keywords::filter=severity >= info,
     keywords::rotation_size = 10 * 1024 * 1024,
     keywords::auto_flush=true
     );

  log_type log;
  BOOST_LOG_SEV(log, trace) << "A trace severity message: " << trace;
  BOOST_LOG_SEV(log, debug) << "A debug severity message: " << debug;
  BOOST_LOG_SEV(log, info) << "An informational severity message: " << info;
  BOOST_LOG_SEV(log, warning) << "A warning severity message: " << warning;
  BOOST_LOG_SEV(log, error) << "An error severity message: " << error;
  BOOST_LOG_SEV(log, fatal) << "A fatal severity message: " << fatal;
}
