/*
 * Copyright (c) 2016,2017 Brook Milligan.
 *
 * This file is part of the GraphModel Library.
 *
 *     The GraphModel Library is free software: you can redistribute
 *     it and/or modify it under the terms of the GNU General Public
 *     License as published by the Free Software Foundation, either
 *     version 3 of the License, or (at your option) any later
 *     version.
 *
 *     The GraphModel Library is distributed in the hope that it will
 *     be useful, but WITHOUT ANY WARRANTY; without even the implied
 *     warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 *     PURPOSE.  See the GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public
 *     License along with the GraphModel Library.  If not, see
 *     <http://www.gnu.org/licenses/>.
 */

/// \file

#define BOOST_TEST_MODULE type_traits_is_rvalue

#ifdef INCLUDE_UNIT_TEST_HEADERS
#include <boost/test/included/unit_test.hpp>
#else
#include <boost/test/unit_test.hpp>
#endif

template < typename ... T > class XXX;

#include <type_traits/is_rvalue.hpp>

class rvalue {};

struct C
{
  void member_function () const {}
  int member_object;
};

void deduce_type (...) {}

template < typename T
	   , typename = std::enable_if_t<type_traits::is_rvalue<T>::value>
	   >
rvalue deduce_type (T&&) {}

int f () { return 0; }

int& g () { static int i; return i; }

BOOST_AUTO_TEST_CASE(test_is_rvalue)
{
  int i;
  C c;

  static_assert(std::is_same<decltype(deduce_type(42)),rvalue>::value,"");
  static_assert(std::is_same<decltype(deduce_type(int())),rvalue>::value,"");
  static_assert(std::is_same<decltype(deduce_type(std::move(i))),rvalue>::value,"");
  static_assert(std::is_same<decltype(deduce_type(f())),rvalue>::value,"");
  static_assert(std::is_same<decltype(deduce_type(i+i)),rvalue>::value,"");
  static_assert(std::is_same<decltype(deduce_type(i<i)),rvalue>::value,"");
  static_assert(std::is_same<decltype(deduce_type(i++)),rvalue>::value,"");
  static_assert(std::is_same<decltype(deduce_type(&i)),rvalue>::value,"");
  static_assert(std::is_same<decltype(deduce_type(&c.member_object)),rvalue>::value,"");
}

BOOST_AUTO_TEST_CASE(test_is_not_rvalue)
{
  int i;
  int array [1];
  int& ref = i;
  int const& cref = i;
  C c;

  static_assert(!std::is_same<decltype(deduce_type(i)),rvalue>::value,"");
  static_assert(!std::is_same<decltype(deduce_type(ref)),rvalue>::value,"");
  static_assert(!std::is_same<decltype(deduce_type(cref)),rvalue>::value,"");
  static_assert(!std::is_same<decltype(deduce_type(g())),rvalue>::value,"");
  static_assert(!std::is_same<decltype(deduce_type(array[0])),rvalue>::value,"");
  static_assert(!std::is_same<decltype(deduce_type(c.member_object)),rvalue>::value,"");
  static_assert(!std::is_same<decltype(deduce_type("42")),rvalue>::value,"");
  static_assert(!std::is_same<decltype(deduce_type(++i)),rvalue>::value,"");
}
